import sys, time

__all__ = ['ProgressReporter']

def time_rep(t):
    '''
    Textual representation of time t given in seconds
    '''
    t = int(t)
    if t < 60:
        return str(t) + 's'
    secs = t % 60
    mins = t // 60
    if mins < 60:
        return str(mins) + 'm ' + str(secs) + 's'
    mins = mins % 60
    hours = t // (60 * 60)
    if hours < 24:
        return str(hours) + 'h ' + str(mins) + 'm ' + str(secs) + 's'
    hours = hours % 24
    days = t // (60 * 60 * 24)
    return str(days) + 'd ' + str(hours) + 'h ' + str(mins) + 'm ' + str(secs) + 's'

def make_text_report(elapsed, complete):
    s = str(int(100 * complete)) + '% complete, '
    s += time_rep(elapsed) + ' elapsed'
    if complete > .001:
        remtime = elapsed / complete - elapsed
        s += ', approximately ' + time_rep(remtime) + ' remaining.'
    else:
        s += '.'
    return s

def build_text_reporter(output_stream):
    def text_report(elapsed, complete):
        s = make_text_report(elapsed, complete) + '\n'
        output_stream.write(s)
        output_stream.flush()
    return text_report


class ProgressReporter(object):
    '''
    Standard text and graphical progress reports
    
    Initialised with arguments:
    
    ``report``
        
        Can be one of the following strings:
    
        ``'print'``, ``'text'``, ``'stdout'``
            Reports progress to standard console.
        ``'stderr'``
            Reports progress to error console.
        ``'graphical'``, ``'tkinter'``
            A simple graphical progress bar using Tkinter.
        
        Alternatively, it can be any output stream in which
        case text reports will be sent to it, or a custom callback function
        ``report(elapsed, complete)`` taking arguments ``elapsed``
        the amount of time that has passed and ``complete`` the fraction of
        the computation finished.
    
    ``period``
        How often reports should be generated in seconds.
    
    Methods:
    
    .. method:: start()
    
        Call at the beginning of a task to start timing it.
    
    .. method:: finish()
    
        Call at the end of a task to finish timing it. Note that
        with the Tkinter class, if you do not call this it will
        stop the Python script from finishing, stopping memory
        from being freed up.
    
    .. method:: update(complete)
    
        Call with the fraction of the task (or subtask if
        ``subtask()`` has been called) completed, between
        0 and 1.
        
    .. method:: subtask(complete, tasksize)
    
        After calling ``subtask(complete, tasksize)``,
        subsequent calls to update will report progress
        between a fraction ``complete`` and ``complete+tasksize``
        of the total task. ``complete`` represents the amount
        of the total task completed at the beginning of the
        task, and ``tasksize`` the size of the subtask as a
        proportion of the whole task.
        
    .. method:: equal_subtask(tasknum, numtasks)
    
        If a task can be divided into ``numtasks`` equally
        sized subtasks, you can use this method instead of
        ``subtask``, where ``tasknum`` is the number of
        the subtask about to start.
    '''
    def __init__(self, report, period=10.0):
        self.period = float(period)
        self.report = get_reporter(report)
        self.start() # just in case the user forgets to call start()

    def start(self):
        self.start_time = time.time()
        self.next_report_time = self.start_time + self.period
        self.subtask_complete = 0.0
        self.subtask_size = 1.0

    def finish(self):
        self.update(1)

    def subtask(self, complete, tasksize):
        self.subtask_complete = complete
        self.subtask_size = tasksize

    def equal_subtask(self, tasknum, numtasks):
        self.subtask(float(tasknum) / float(numtasks), 1. / numtasks)

    def update(self, complete):
        cur_time = time.time()
        totalcomplete = self.subtask_complete + complete * self.subtask_size
        if cur_time > self.next_report_time or totalcomplete == 1.0 or totalcomplete == 1:
            self.next_report_time = cur_time + self.period
            elapsed = time.time() - self.start_time
            self.report(elapsed, totalcomplete)

def get_reporter(report):
    if report == 'print' or report == 'text' or report == 'stdout':
        report = build_text_reporter(sys.stdout)
    elif report == 'stderr':
        report = build_text_reporter(sys.stderr)
    elif hasattr(report, 'write') and hasattr(report, 'flush'):
        report = build_text_reporter(report)
    elif report == 'graphical' or report == 'tkinter':
        import Tkinter
        class ProgressBar(object):
            '''
            Adapted from: http://code.activestate.com/recipes/492230/
            '''
            # Create Progress Bar
            def __init__(self, width, height):
                self.__root = Tkinter.Tk()
                self.__root.resizable(False, False)
                self.__root.title('Progress Bar')
                self.__canvas = Tkinter.Canvas(self.__root, width=width, height=height)
                self.__canvas.grid()
                self.__width = width
                self.__height = height
            # Open Progress Bar
            def open(self):
                self.__root.deiconify()
            # Close Progress Bar
            def close(self):
                self.__root.withdraw()
            # Update Progress Bar
            def update(self, ratio, newtitle=None):
                self.__canvas.delete(Tkinter.ALL)
                self.__canvas.create_rectangle(0, 0, self.__width * ratio, \
                                               self.__height, fill='blue')
                if newtitle is not None:
                    self.__root.title(newtitle)
                self.__root.update()
        pb = ProgressBar(500, 20)
        pb.closed = False
        def report(elapsed, complete):
            try:
                if complete == 1.0 and pb.closed == False:
                    pb.close()
                    pb.closed = True
                else:
                    pb.update(complete, make_text_report(elapsed, complete))
            except Tkinter.TclError:
                # exception handling in the case that the user shuts the window
                pass
    return report

if __name__ == '__main__':
    import time
    report = ProgressReporter('graphical', 0.5)
    report.start()
    time.sleep(1)
    report.update(1. / 3)
    time.sleep(1)
    report.update(2. / 3)
    time.sleep(1)
    report.update(3. / 3)
    time.sleep(1)



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 1.3.33
#
# Don't modify this file, modify the SWIG interface instead.
# This file is compatible with both classic and new-style classes.

import _brianlib
import new
new_instancemethod = new.instancemethod
try:
    _swig_property = property
except NameError:
    pass # Python < 2.2 doesn't have 'property'.
def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'PySwigObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method: return method(self, value)
    if (not static) or hasattr(self, name):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)

def _swig_getattr(self, class_type, name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method: return method(self)
    raise AttributeError, name

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

import types
try:
    _object = types.ObjectType
    _newclass = 1
except AttributeError:
    class _object : pass
    _newclass = 0
del types


class PySwigIterator(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, PySwigIterator, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, PySwigIterator, name)
    def __init__(self): raise AttributeError, "No constructor defined"
    __repr__ = _swig_repr
    __swig_destroy__ = _brianlib.delete_PySwigIterator
    __del__ = lambda self : None;
    def value(*args): return _brianlib.PySwigIterator_value(*args)
    def incr(*args): return _brianlib.PySwigIterator_incr(*args)
    def decr(*args): return _brianlib.PySwigIterator_decr(*args)
    def distance(*args): return _brianlib.PySwigIterator_distance(*args)
    def equal(*args): return _brianlib.PySwigIterator_equal(*args)
    def copy(*args): return _brianlib.PySwigIterator_copy(*args)
    def next(*args): return _brianlib.PySwigIterator_next(*args)
    def previous(*args): return _brianlib.PySwigIterator_previous(*args)
    def advance(*args): return _brianlib.PySwigIterator_advance(*args)
    def __eq__(*args): return _brianlib.PySwigIterator___eq__(*args)
    def __ne__(*args): return _brianlib.PySwigIterator___ne__(*args)
    def __iadd__(*args): return _brianlib.PySwigIterator___iadd__(*args)
    def __isub__(*args): return _brianlib.PySwigIterator___isub__(*args)
    def __add__(*args): return _brianlib.PySwigIterator___add__(*args)
    def __sub__(*args): return _brianlib.PySwigIterator___sub__(*args)
    def __iter__(self): return self
PySwigIterator_swigregister = _brianlib.PySwigIterator_swigregister
PySwigIterator_swigregister(PySwigIterator)

class SpikeList(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, SpikeList, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, SpikeList, name)
    __repr__ = _swig_repr
    def iterator(*args): return _brianlib.SpikeList_iterator(*args)
    def __iter__(self): return self.iterator()
    def __nonzero__(*args): return _brianlib.SpikeList___nonzero__(*args)
    def __len__(*args): return _brianlib.SpikeList___len__(*args)
    def pop(*args): return _brianlib.SpikeList_pop(*args)
    def __getslice__(*args): return _brianlib.SpikeList___getslice__(*args)
    def __setslice__(*args): return _brianlib.SpikeList___setslice__(*args)
    def __delslice__(*args): return _brianlib.SpikeList___delslice__(*args)
    def __delitem__(*args): return _brianlib.SpikeList___delitem__(*args)
    def __getitem__(*args): return _brianlib.SpikeList___getitem__(*args)
    def __setitem__(*args): return _brianlib.SpikeList___setitem__(*args)
    def append(*args): return _brianlib.SpikeList_append(*args)
    def empty(*args): return _brianlib.SpikeList_empty(*args)
    def size(*args): return _brianlib.SpikeList_size(*args)
    def clear(*args): return _brianlib.SpikeList_clear(*args)
    def swap(*args): return _brianlib.SpikeList_swap(*args)
    def get_allocator(*args): return _brianlib.SpikeList_get_allocator(*args)
    def begin(*args): return _brianlib.SpikeList_begin(*args)
    def end(*args): return _brianlib.SpikeList_end(*args)
    def rbegin(*args): return _brianlib.SpikeList_rbegin(*args)
    def rend(*args): return _brianlib.SpikeList_rend(*args)
    def pop_back(*args): return _brianlib.SpikeList_pop_back(*args)
    def erase(*args): return _brianlib.SpikeList_erase(*args)
    def __init__(self, *args):
        this = _brianlib.new_SpikeList(*args)
        try: self.this.append(this)
        except: self.this = this
    def push_back(*args): return _brianlib.SpikeList_push_back(*args)
    def front(*args): return _brianlib.SpikeList_front(*args)
    def back(*args): return _brianlib.SpikeList_back(*args)
    def assign(*args): return _brianlib.SpikeList_assign(*args)
    def resize(*args): return _brianlib.SpikeList_resize(*args)
    def insert(*args): return _brianlib.SpikeList_insert(*args)
    def pop_front(*args): return _brianlib.SpikeList_pop_front(*args)
    def push_front(*args): return _brianlib.SpikeList_push_front(*args)
    def reverse(*args): return _brianlib.SpikeList_reverse(*args)
    __swig_destroy__ = _brianlib.delete_SpikeList
    __del__ = lambda self : None;
SpikeList_swigregister = _brianlib.SpikeList_swigregister
SpikeList_swigregister(SpikeList)

class VectorDouble(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, VectorDouble, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, VectorDouble, name)
    __repr__ = _swig_repr
    def iterator(*args): return _brianlib.VectorDouble_iterator(*args)
    def __iter__(self): return self.iterator()
    def __nonzero__(*args): return _brianlib.VectorDouble___nonzero__(*args)
    def __len__(*args): return _brianlib.VectorDouble___len__(*args)
    def pop(*args): return _brianlib.VectorDouble_pop(*args)
    def __getslice__(*args): return _brianlib.VectorDouble___getslice__(*args)
    def __setslice__(*args): return _brianlib.VectorDouble___setslice__(*args)
    def __delslice__(*args): return _brianlib.VectorDouble___delslice__(*args)
    def __delitem__(*args): return _brianlib.VectorDouble___delitem__(*args)
    def __getitem__(*args): return _brianlib.VectorDouble___getitem__(*args)
    def __setitem__(*args): return _brianlib.VectorDouble___setitem__(*args)
    def append(*args): return _brianlib.VectorDouble_append(*args)
    def empty(*args): return _brianlib.VectorDouble_empty(*args)
    def size(*args): return _brianlib.VectorDouble_size(*args)
    def clear(*args): return _brianlib.VectorDouble_clear(*args)
    def swap(*args): return _brianlib.VectorDouble_swap(*args)
    def get_allocator(*args): return _brianlib.VectorDouble_get_allocator(*args)
    def begin(*args): return _brianlib.VectorDouble_begin(*args)
    def end(*args): return _brianlib.VectorDouble_end(*args)
    def rbegin(*args): return _brianlib.VectorDouble_rbegin(*args)
    def rend(*args): return _brianlib.VectorDouble_rend(*args)
    def pop_back(*args): return _brianlib.VectorDouble_pop_back(*args)
    def erase(*args): return _brianlib.VectorDouble_erase(*args)
    def __init__(self, *args):
        this = _brianlib.new_VectorDouble(*args)
        try: self.this.append(this)
        except: self.this = this
    def push_back(*args): return _brianlib.VectorDouble_push_back(*args)
    def front(*args): return _brianlib.VectorDouble_front(*args)
    def back(*args): return _brianlib.VectorDouble_back(*args)
    def assign(*args): return _brianlib.VectorDouble_assign(*args)
    def resize(*args): return _brianlib.VectorDouble_resize(*args)
    def insert(*args): return _brianlib.VectorDouble_insert(*args)
    def reserve(*args): return _brianlib.VectorDouble_reserve(*args)
    def capacity(*args): return _brianlib.VectorDouble_capacity(*args)
    __swig_destroy__ = _brianlib.delete_VectorDouble
    __del__ = lambda self : None;
VectorDouble_swigregister = _brianlib.VectorDouble_swigregister
VectorDouble_swigregister(VectorDouble)

class NeuronGroup(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, NeuronGroup, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, NeuronGroup, name)
    __repr__ = _swig_repr
    __swig_setmethods__["S"] = _brianlib.NeuronGroup_S_set
    __swig_getmethods__["S"] = _brianlib.NeuronGroup_S_get
    if _newclass:S = _swig_property(_brianlib.NeuronGroup_S_get, _brianlib.NeuronGroup_S_set)
    __swig_setmethods__["num_vars"] = _brianlib.NeuronGroup_num_vars_set
    __swig_getmethods__["num_vars"] = _brianlib.NeuronGroup_num_vars_get
    if _newclass:num_vars = _swig_property(_brianlib.NeuronGroup_num_vars_get, _brianlib.NeuronGroup_num_vars_set)
    __swig_setmethods__["num_neurons"] = _brianlib.NeuronGroup_num_neurons_set
    __swig_getmethods__["num_neurons"] = _brianlib.NeuronGroup_num_neurons_get
    if _newclass:num_neurons = _swig_property(_brianlib.NeuronGroup_num_neurons_get, _brianlib.NeuronGroup_num_neurons_set)
    __swig_setmethods__["S_m"] = _brianlib.NeuronGroup_S_m_set
    __swig_getmethods__["S_m"] = _brianlib.NeuronGroup_S_m_get
    if _newclass:S_m = _swig_property(_brianlib.NeuronGroup_S_m_get, _brianlib.NeuronGroup_S_m_set)
    __swig_setmethods__["su"] = _brianlib.NeuronGroup_su_set
    __swig_getmethods__["su"] = _brianlib.NeuronGroup_su_get
    if _newclass:su = _swig_property(_brianlib.NeuronGroup_su_get, _brianlib.NeuronGroup_su_set)
    __swig_setmethods__["thr"] = _brianlib.NeuronGroup_thr_set
    __swig_getmethods__["thr"] = _brianlib.NeuronGroup_thr_get
    if _newclass:thr = _swig_property(_brianlib.NeuronGroup_thr_get, _brianlib.NeuronGroup_thr_set)
    __swig_setmethods__["resetobj"] = _brianlib.NeuronGroup_resetobj_set
    __swig_getmethods__["resetobj"] = _brianlib.NeuronGroup_resetobj_get
    if _newclass:resetobj = _swig_property(_brianlib.NeuronGroup_resetobj_get, _brianlib.NeuronGroup_resetobj_set)
    __swig_setmethods__["LS"] = _brianlib.NeuronGroup_LS_set
    __swig_getmethods__["LS"] = _brianlib.NeuronGroup_LS_get
    if _newclass:LS = _swig_property(_brianlib.NeuronGroup_LS_get, _brianlib.NeuronGroup_LS_set)
    __swig_setmethods__["spikesarray"] = _brianlib.NeuronGroup_spikesarray_set
    __swig_getmethods__["spikesarray"] = _brianlib.NeuronGroup_spikesarray_get
    if _newclass:spikesarray = _swig_property(_brianlib.NeuronGroup_spikesarray_get, _brianlib.NeuronGroup_spikesarray_set)
    __swig_setmethods__["owner"] = _brianlib.NeuronGroup_owner_set
    __swig_getmethods__["owner"] = _brianlib.NeuronGroup_owner_get
    if _newclass:owner = _swig_property(_brianlib.NeuronGroup_owner_get, _brianlib.NeuronGroup_owner_set)
    __swig_setmethods__["origin"] = _brianlib.NeuronGroup_origin_set
    __swig_getmethods__["origin"] = _brianlib.NeuronGroup_origin_get
    if _newclass:origin = _swig_property(_brianlib.NeuronGroup_origin_get, _brianlib.NeuronGroup_origin_set)
    def __init__(self, *args):
        this = _brianlib.new_NeuronGroup(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _brianlib.delete_NeuronGroup
    __del__ = lambda self : None;
    def get_spikes(*args): return _brianlib.NeuronGroup_get_spikes(*args)
    def update(*args): return _brianlib.NeuronGroup_update(*args)
    def reset(*args): return _brianlib.NeuronGroup_reset(*args)
NeuronGroup_swigregister = _brianlib.NeuronGroup_swigregister
NeuronGroup_swigregister(NeuronGroup)

class StateUpdater(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, StateUpdater, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, StateUpdater, name)
    __repr__ = _swig_repr
    def __call__(*args): return _brianlib.StateUpdater___call__(*args)
    def __init__(self, *args):
        this = _brianlib.new_StateUpdater(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _brianlib.delete_StateUpdater
    __del__ = lambda self : None;
StateUpdater_swigregister = _brianlib.StateUpdater_swigregister
StateUpdater_swigregister(StateUpdater)

class LinearStateUpdater(StateUpdater):
    __swig_setmethods__ = {}
    for _s in [StateUpdater]: __swig_setmethods__.update(getattr(_s, '__swig_setmethods__', {}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, LinearStateUpdater, name, value)
    __swig_getmethods__ = {}
    for _s in [StateUpdater]: __swig_getmethods__.update(getattr(_s, '__swig_getmethods__', {}))
    __getattr__ = lambda self, name: _swig_getattr(self, LinearStateUpdater, name)
    __swig_setmethods__["M"] = _brianlib.LinearStateUpdater_M_set
    __swig_getmethods__["M"] = _brianlib.LinearStateUpdater_M_get
    if _newclass:M = _swig_property(_brianlib.LinearStateUpdater_M_get, _brianlib.LinearStateUpdater_M_set)
    __swig_setmethods__["M_n"] = _brianlib.LinearStateUpdater_M_n_set
    __swig_getmethods__["M_n"] = _brianlib.LinearStateUpdater_M_n_get
    if _newclass:M_n = _swig_property(_brianlib.LinearStateUpdater_M_n_get, _brianlib.LinearStateUpdater_M_n_set)
    __swig_setmethods__["M_m"] = _brianlib.LinearStateUpdater_M_m_set
    __swig_getmethods__["M_m"] = _brianlib.LinearStateUpdater_M_m_get
    if _newclass:M_m = _swig_property(_brianlib.LinearStateUpdater_M_m_get, _brianlib.LinearStateUpdater_M_m_set)
    __swig_setmethods__["b"] = _brianlib.LinearStateUpdater_b_set
    __swig_getmethods__["b"] = _brianlib.LinearStateUpdater_b_get
    if _newclass:b = _swig_property(_brianlib.LinearStateUpdater_b_get, _brianlib.LinearStateUpdater_b_set)
    __swig_setmethods__["b_n"] = _brianlib.LinearStateUpdater_b_n_set
    __swig_getmethods__["b_n"] = _brianlib.LinearStateUpdater_b_n_get
    if _newclass:b_n = _swig_property(_brianlib.LinearStateUpdater_b_n_get, _brianlib.LinearStateUpdater_b_n_set)
    def __init__(self, *args):
        this = _brianlib.new_LinearStateUpdater(*args)
        try: self.this.append(this)
        except: self.this = this
    def __call__(*args): return _brianlib.LinearStateUpdater___call__(*args)
    def __repr__(*args): return _brianlib.LinearStateUpdater___repr__(*args)
    __swig_destroy__ = _brianlib.delete_LinearStateUpdater
    __del__ = lambda self : None;
LinearStateUpdater_swigregister = _brianlib.LinearStateUpdater_swigregister
LinearStateUpdater_swigregister(LinearStateUpdater)

class Threshold(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Threshold, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Threshold, name)
    __repr__ = _swig_repr
    __swig_setmethods__["value"] = _brianlib.Threshold_value_set
    __swig_getmethods__["value"] = _brianlib.Threshold_value_get
    if _newclass:value = _swig_property(_brianlib.Threshold_value_get, _brianlib.Threshold_value_set)
    __swig_setmethods__["state"] = _brianlib.Threshold_state_set
    __swig_getmethods__["state"] = _brianlib.Threshold_state_get
    if _newclass:state = _swig_property(_brianlib.Threshold_state_get, _brianlib.Threshold_state_set)
    def __init__(self, *args):
        this = _brianlib.new_Threshold(*args)
        try: self.this.append(this)
        except: self.this = this
    def __call__(*args): return _brianlib.Threshold___call__(*args)
    __swig_destroy__ = _brianlib.delete_Threshold
    __del__ = lambda self : None;
Threshold_swigregister = _brianlib.Threshold_swigregister
Threshold_swigregister(Threshold)

class ResetBase(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, ResetBase, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, ResetBase, name)
    __repr__ = _swig_repr
    def __call__(*args): return _brianlib.ResetBase___call__(*args)
    def __init__(self, *args):
        this = _brianlib.new_ResetBase(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _brianlib.delete_ResetBase
    __del__ = lambda self : None;
ResetBase_swigregister = _brianlib.ResetBase_swigregister
ResetBase_swigregister(ResetBase)

class Reset(ResetBase):
    __swig_setmethods__ = {}
    for _s in [ResetBase]: __swig_setmethods__.update(getattr(_s, '__swig_setmethods__', {}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, Reset, name, value)
    __swig_getmethods__ = {}
    for _s in [ResetBase]: __swig_getmethods__.update(getattr(_s, '__swig_getmethods__', {}))
    __getattr__ = lambda self, name: _swig_getattr(self, Reset, name)
    __repr__ = _swig_repr
    __swig_setmethods__["value"] = _brianlib.Reset_value_set
    __swig_getmethods__["value"] = _brianlib.Reset_value_get
    if _newclass:value = _swig_property(_brianlib.Reset_value_get, _brianlib.Reset_value_set)
    __swig_setmethods__["state"] = _brianlib.Reset_state_set
    __swig_getmethods__["state"] = _brianlib.Reset_state_get
    if _newclass:state = _swig_property(_brianlib.Reset_state_get, _brianlib.Reset_state_set)
    def __init__(self, *args):
        this = _brianlib.new_Reset(*args)
        try: self.this.append(this)
        except: self.this = this
    def __call__(*args): return _brianlib.Reset___call__(*args)
    __swig_destroy__ = _brianlib.delete_Reset
    __del__ = lambda self : None;
Reset_swigregister = _brianlib.Reset_swigregister
Reset_swigregister(Reset)

class Refractoriness(Reset):
    __swig_setmethods__ = {}
    for _s in [Reset]: __swig_setmethods__.update(getattr(_s, '__swig_setmethods__', {}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, Refractoriness, name, value)
    __swig_getmethods__ = {}
    for _s in [Reset]: __swig_getmethods__.update(getattr(_s, '__swig_getmethods__', {}))
    __getattr__ = lambda self, name: _swig_getattr(self, Refractoriness, name)
    __repr__ = _swig_repr
    __swig_setmethods__["period"] = _brianlib.Refractoriness_period_set
    __swig_getmethods__["period"] = _brianlib.Refractoriness_period_get
    if _newclass:period = _swig_property(_brianlib.Refractoriness_period_get, _brianlib.Refractoriness_period_set)
    def __init__(self, *args):
        this = _brianlib.new_Refractoriness(*args)
        try: self.this.append(this)
        except: self.this = this
    def __call__(*args): return _brianlib.Refractoriness___call__(*args)
    __swig_destroy__ = _brianlib.delete_Refractoriness
    __del__ = lambda self : None;
Refractoriness_swigregister = _brianlib.Refractoriness_swigregister
Refractoriness_swigregister(Refractoriness)

class NetworkOperation(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, NetworkOperation, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, NetworkOperation, name)
    __repr__ = _swig_repr
    def __call__(*args): return _brianlib.NetworkOperation___call__(*args)
    def __init__(self, *args):
        this = _brianlib.new_NetworkOperation(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _brianlib.delete_NetworkOperation
    __del__ = lambda self : None;
NetworkOperation_swigregister = _brianlib.NetworkOperation_swigregister
NetworkOperation_swigregister(NetworkOperation)

class StateMonitor(NetworkOperation):
    __swig_setmethods__ = {}
    for _s in [NetworkOperation]: __swig_setmethods__.update(getattr(_s, '__swig_setmethods__', {}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, StateMonitor, name, value)
    __swig_getmethods__ = {}
    for _s in [NetworkOperation]: __swig_getmethods__.update(getattr(_s, '__swig_getmethods__', {}))
    __getattr__ = lambda self, name: _swig_getattr(self, StateMonitor, name)
    __repr__ = _swig_repr
    __swig_setmethods__["group"] = _brianlib.StateMonitor_group_set
    __swig_getmethods__["group"] = _brianlib.StateMonitor_group_get
    if _newclass:group = _swig_property(_brianlib.StateMonitor_group_get, _brianlib.StateMonitor_group_set)
    __swig_setmethods__["state"] = _brianlib.StateMonitor_state_set
    __swig_getmethods__["state"] = _brianlib.StateMonitor_state_get
    if _newclass:state = _swig_property(_brianlib.StateMonitor_state_get, _brianlib.StateMonitor_state_set)
    __swig_setmethods__["values"] = _brianlib.StateMonitor_values_set
    __swig_getmethods__["values"] = _brianlib.StateMonitor_values_get
    if _newclass:values = _swig_property(_brianlib.StateMonitor_values_get, _brianlib.StateMonitor_values_set)
    def __init__(self, *args):
        this = _brianlib.new_StateMonitor(*args)
        try: self.this.append(this)
        except: self.this = this
    def __call__(*args): return _brianlib.StateMonitor___call__(*args)
    def __getitem__(*args): return _brianlib.StateMonitor___getitem__(*args)
    __swig_destroy__ = _brianlib.delete_StateMonitor
    __del__ = lambda self : None;
StateMonitor_swigregister = _brianlib.StateMonitor_swigregister
StateMonitor_swigregister(StateMonitor)

class Network(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Network, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Network, name)
    __repr__ = _swig_repr
    __swig_setmethods__["groups"] = _brianlib.Network_groups_set
    __swig_getmethods__["groups"] = _brianlib.Network_groups_get
    if _newclass:groups = _swig_property(_brianlib.Network_groups_get, _brianlib.Network_groups_set)
    __swig_setmethods__["operations"] = _brianlib.Network_operations_set
    __swig_getmethods__["operations"] = _brianlib.Network_operations_get
    if _newclass:operations = _swig_property(_brianlib.Network_operations_get, _brianlib.Network_operations_set)
    __swig_setmethods__["connections"] = _brianlib.Network_connections_set
    __swig_getmethods__["connections"] = _brianlib.Network_connections_get
    if _newclass:connections = _swig_property(_brianlib.Network_connections_get, _brianlib.Network_connections_set)
    def add(*args): return _brianlib.Network_add(*args)
    def update(*args): return _brianlib.Network_update(*args)
    def run(*args): return _brianlib.Network_run(*args)
    def __init__(self, *args):
        this = _brianlib.new_Network(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _brianlib.delete_Network
    __del__ = lambda self : None;
Network_swigregister = _brianlib.Network_swigregister
Network_swigregister(Network)

class CircularVector(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, CircularVector, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, CircularVector, name)
    __swig_setmethods__["X"] = _brianlib.CircularVector_X_set
    __swig_getmethods__["X"] = _brianlib.CircularVector_X_get
    if _newclass:X = _swig_property(_brianlib.CircularVector_X_get, _brianlib.CircularVector_X_set)
    __swig_setmethods__["cursor"] = _brianlib.CircularVector_cursor_set
    __swig_getmethods__["cursor"] = _brianlib.CircularVector_cursor_get
    if _newclass:cursor = _swig_property(_brianlib.CircularVector_cursor_get, _brianlib.CircularVector_cursor_set)
    __swig_setmethods__["n"] = _brianlib.CircularVector_n_set
    __swig_getmethods__["n"] = _brianlib.CircularVector_n_get
    if _newclass:n = _swig_property(_brianlib.CircularVector_n_get, _brianlib.CircularVector_n_set)
    __swig_setmethods__["retarray"] = _brianlib.CircularVector_retarray_set
    __swig_getmethods__["retarray"] = _brianlib.CircularVector_retarray_get
    if _newclass:retarray = _swig_property(_brianlib.CircularVector_retarray_get, _brianlib.CircularVector_retarray_set)
    def __init__(self, *args):
        this = _brianlib.new_CircularVector(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _brianlib.delete_CircularVector
    __del__ = lambda self : None;
    def reinit(*args): return _brianlib.CircularVector_reinit(*args)
    def advance(*args): return _brianlib.CircularVector_advance(*args)
    def __len__(*args): return _brianlib.CircularVector___len__(*args)
    def __getitem__(*args): return _brianlib.CircularVector___getitem__(*args)
    def __setitem__(*args): return _brianlib.CircularVector___setitem__(*args)
    def __getslice__(*args): return _brianlib.CircularVector___getslice__(*args)
    def get_conditional(*args): return _brianlib.CircularVector_get_conditional(*args)
    def __setslice__(*args): return _brianlib.CircularVector___setslice__(*args)
    def __repr__(*args): return _brianlib.CircularVector___repr__(*args)
    def __str__(*args): return _brianlib.CircularVector___str__(*args)
CircularVector_swigregister = _brianlib.CircularVector_swigregister
CircularVector_swigregister(CircularVector)

class SpikeContainer(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, SpikeContainer, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, SpikeContainer, name)
    __swig_setmethods__["S"] = _brianlib.SpikeContainer_S_set
    __swig_getmethods__["S"] = _brianlib.SpikeContainer_S_get
    if _newclass:S = _swig_property(_brianlib.SpikeContainer_S_get, _brianlib.SpikeContainer_S_set)
    __swig_setmethods__["ind"] = _brianlib.SpikeContainer_ind_set
    __swig_getmethods__["ind"] = _brianlib.SpikeContainer_ind_get
    if _newclass:ind = _swig_property(_brianlib.SpikeContainer_ind_get, _brianlib.SpikeContainer_ind_set)
    def __init__(self, *args):
        this = _brianlib.new_SpikeContainer(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _brianlib.delete_SpikeContainer
    __del__ = lambda self : None;
    def reinit(*args): return _brianlib.SpikeContainer_reinit(*args)
    def push(*args): return _brianlib.SpikeContainer_push(*args)
    def lastspikes(*args): return _brianlib.SpikeContainer_lastspikes(*args)
    def __getitem__(*args): return _brianlib.SpikeContainer___getitem__(*args)
    def get_spikes(*args): return _brianlib.SpikeContainer_get_spikes(*args)
    def __getslice__(*args): return _brianlib.SpikeContainer___getslice__(*args)
    def __repr__(*args): return _brianlib.SpikeContainer___repr__(*args)
    def __str__(*args): return _brianlib.SpikeContainer___str__(*args)
SpikeContainer_swigregister = _brianlib.SpikeContainer_swigregister
SpikeContainer_swigregister(SpikeContainer)

class ConnectionMatrix(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, ConnectionMatrix, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, ConnectionMatrix, name)
    __repr__ = _swig_repr
    def add_row(*args): return _brianlib.ConnectionMatrix_add_row(*args)
    def add_rows(*args): return _brianlib.ConnectionMatrix_add_rows(*args)
    def __init__(self, *args):
        this = _brianlib.new_ConnectionMatrix(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _brianlib.delete_ConnectionMatrix
    __del__ = lambda self : None;
ConnectionMatrix_swigregister = _brianlib.ConnectionMatrix_swigregister
ConnectionMatrix_swigregister(ConnectionMatrix)

class DenseConnectionMatrix(ConnectionMatrix):
    __swig_setmethods__ = {}
    for _s in [ConnectionMatrix]: __swig_setmethods__.update(getattr(_s, '__swig_setmethods__', {}))
    __setattr__ = lambda self, name, value: _swig_setattr(self, DenseConnectionMatrix, name, value)
    __swig_getmethods__ = {}
    for _s in [ConnectionMatrix]: __swig_getmethods__.update(getattr(_s, '__swig_getmethods__', {}))
    __getattr__ = lambda self, name: _swig_getattr(self, DenseConnectionMatrix, name)
    __repr__ = _swig_repr
    __swig_setmethods__["W"] = _brianlib.DenseConnectionMatrix_W_set
    __swig_getmethods__["W"] = _brianlib.DenseConnectionMatrix_W_get
    if _newclass:W = _swig_property(_brianlib.DenseConnectionMatrix_W_get, _brianlib.DenseConnectionMatrix_W_set)
    __swig_setmethods__["n"] = _brianlib.DenseConnectionMatrix_n_set
    __swig_getmethods__["n"] = _brianlib.DenseConnectionMatrix_n_get
    if _newclass:n = _swig_property(_brianlib.DenseConnectionMatrix_n_get, _brianlib.DenseConnectionMatrix_n_set)
    __swig_setmethods__["m"] = _brianlib.DenseConnectionMatrix_m_set
    __swig_getmethods__["m"] = _brianlib.DenseConnectionMatrix_m_get
    if _newclass:m = _swig_property(_brianlib.DenseConnectionMatrix_m_get, _brianlib.DenseConnectionMatrix_m_set)
    def __init__(self, *args):
        this = _brianlib.new_DenseConnectionMatrix(*args)
        try: self.this.append(this)
        except: self.this = this
    def add_row(*args): return _brianlib.DenseConnectionMatrix_add_row(*args)
    __swig_destroy__ = _brianlib.delete_DenseConnectionMatrix
    __del__ = lambda self : None;
DenseConnectionMatrix_swigregister = _brianlib.DenseConnectionMatrix_swigregister
DenseConnectionMatrix_swigregister(DenseConnectionMatrix)

class Connection(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, Connection, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, Connection, name)
    __repr__ = _swig_repr
    __swig_setmethods__["source"] = _brianlib.Connection_source_set
    __swig_getmethods__["source"] = _brianlib.Connection_source_get
    if _newclass:source = _swig_property(_brianlib.Connection_source_get, _brianlib.Connection_source_set)
    __swig_setmethods__["target"] = _brianlib.Connection_target_set
    __swig_getmethods__["target"] = _brianlib.Connection_target_get
    if _newclass:target = _swig_property(_brianlib.Connection_target_get, _brianlib.Connection_target_set)
    __swig_setmethods__["connmat"] = _brianlib.Connection_connmat_set
    __swig_getmethods__["connmat"] = _brianlib.Connection_connmat_get
    if _newclass:connmat = _swig_property(_brianlib.Connection_connmat_get, _brianlib.Connection_connmat_set)
    __swig_setmethods__["state"] = _brianlib.Connection_state_set
    __swig_getmethods__["state"] = _brianlib.Connection_state_get
    if _newclass:state = _swig_property(_brianlib.Connection_state_get, _brianlib.Connection_state_set)
    __swig_setmethods__["delay"] = _brianlib.Connection_delay_set
    __swig_getmethods__["delay"] = _brianlib.Connection_delay_get
    if _newclass:delay = _swig_property(_brianlib.Connection_delay_get, _brianlib.Connection_delay_set)
    def __init__(self, *args):
        this = _brianlib.new_Connection(*args)
        try: self.this.append(this)
        except: self.this = this
    def propagate(*args): return _brianlib.Connection_propagate(*args)
    def do_propagate(*args): return _brianlib.Connection_do_propagate(*args)
    __swig_destroy__ = _brianlib.delete_Connection
    __del__ = lambda self : None;
Connection_swigregister = _brianlib.Connection_swigregister
Connection_swigregister(Connection)






import copy
from itertools import izip
import itertools
from random import sample
import bisect
from ..units import second, msecond, check_units, DimensionMismatchError
import types
from .. import magic
from ..log import log_warn, log_info, log_debug
from numpy import *
from scipy import sparse, stats, rand, weave, linalg
import scipy
import scipy.sparse
import numpy
from numpy.random import binomial, exponential
import random as pyrandom
from scipy import random as scirandom
from ..utils.approximatecomparisons import is_within_absolute_tolerance
from ..globalprefs import get_global_preference
from ..base import ObjectContainer
from ..stdunits import ms
from operator import isSequenceType

effective_zero = 1e-40

colon_slice = slice(None, None, None)

def todense(x):
    if hasattr(x, 'todense'):
        return x.todense()
    return array(x, copy=False)



'''
Very short example program.
'''
from brian import *
import time

Nplot = 4
N = 4000
Ne = int(0.8 * N)
Ni = N - Ne
T = 200 * ms

eqs = '''
dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
dge/dt = -ge/(5*ms) : volt
dgi/dt = -gi/(10*ms) : volt
'''

P = NeuronGroup(N, model=eqs,
              threshold= -50 * mV, reset= -60 * mV)
P.v = -60 * mV + 10 * mV * rand(len(P))
Pe = P.subgroup(Ne)
Pi = P.subgroup(Ni)

Ce = Connection(Pe, P, 'ge')
Ci = Connection(Pi, P, 'gi')
Ce.connect_random(Pe, P, 0.02, weight=1.62 * mV)
Ci.connect_random(Pi, P, 0.02, weight= -9 * mV)

M = SpikeMonitor(P)
MV = StateMonitor(P, 'v', record=range(Nplot))

start = time.time()
run(T)

print 'Time taken', time.time() - start
print 'Spikes', M.nspikes

raster_plot(M)
figure()
for i in range(Nplot):
    plot(MV[i])
show()



# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
"""
Clocks for the simulator
"""

__docformat__ = "restructuredtext en"

__all__ = ['Clock', 'defaultclock', 'guess_clock', 'define_default_clock', 'reinit_default_clock', 'get_default_clock',
           'EventClock', 'RegularClock', 'FloatClock', 'NaiveClock']

from inspect import stack
from units import *
from globalprefs import *
import magic
from time import time
from numpy import ceil


class Clock(magic.InstanceTracker):
    '''
    An object that holds the simulation time and the time step.
    
    Initialisation arguments:
    
    ``dt``
        The time step of the simulation.
    ``t``
        The current time of the clock.
    ``order``
        If two clocks have the same time, the order of the clock is used to
        resolve which clock is processed first, lower orders first.
    ``makedefaultclock``
        Set to ``True`` to make this clock the default clock.
    
    The times returned by this clock are always off the form ``n*dt+offset``
    for integer ``n`` and float ``dt`` and ``offset``. For example, for a clock
    with ``dt=10*ms``, setting ``t=25*ms`` will set ``n=2`` and ``offset=5*ms``.
    For a clock that uses true float values for ``t`` rather than underlying
    integers, use :class:`FloatClock` (although see the caveats there).
    
    In order to make sure that certain operations happen in the correct
    sequence, you can use the ``order`` attribute, clocks with a lower order
    will be processed first if the time is the same. The condition for two
    clocks to be considered as having the same time is
    ``abs(t1-t2)<epsilon*abs(t1)``, a standard test for equality of floating
    point values. For ordinary clocks based on integer times, the value of
    ``epsilon`` is ``1e-14``, and for float based clocks it is ``1e-8``.
    
    The behaviour of clocks was changed in version 1.3 of Brian, if this is
    causing problems you might try using :class:`FloatClock` or if that doesn't
    solve the problem, :class:`NaiveClock`.
    
    **Methods**
    
    .. method:: reinit([t=0*second])
    
        Reinitialises the clock time to zero (or to your
        specified time).
    
    **Attributes**
    
    .. attribute:: t
                   dt
    
        Current time and time step with units.
    
    **Advanced**
    
    *Attributes*
    
    .. attribute:: end
    
        The time at which the current simulation will end,
        set by the :meth:`Network.run` method.
    
    *Methods*
    
    .. method:: tick()
    
        Advances the clock by one time step.
        
    .. method:: set_t(t)
                set_dt(dt)
                set_end(end)
    
        Set the various parameters.
        
    .. method:: get_duration()
    
        The time until the current simulation ends.
        
    .. method:: set_duration(duration)
    
        Set the time until the current simulation ends.
        
    .. method:: still_running()
    
        Returns a ``bool`` to indicate whether the current
        simulation is still running.
    
    For reasons of efficiency, we recommend using the methods
    :meth:`tick`, :meth:`set_duration` and :meth:`still_running`
    (which bypass unit checking internally).
    '''
    @check_units(dt=second, t=second)
    def __init__(self, dt=0.1*msecond, t=0*msecond, order=0,
                 makedefaultclock=False):
        self._gridoffset = 0.0
        self.__dt = 1
        self.dt = dt
        self.t = t
        #self.__t = int(t / dt)
        self.__end = 0
        self.order = order
        if not exists_global_preference('defaultclock') or makedefaultclock:
            set_global_preferences(defaultclock=self)

    @check_units(t=second)
    def reinit(self, t=0 * msecond):
        self.__t = int(float(t) / self._dt)
        self._gridoffset = 0.0

    def __repr__(self):
        return 'Clock: t = ' + str(self.t) + ', dt = ' + str(self.dt)
    __str__ = __repr__

    def tick(self):
        self.__t += self.__dt

    @check_units(t=second)
    def set_t(self, t):
        self.__t = int(float(t) / self._dt)
        self._gridoffset = float(t)-self.__t*self._dt
        #self.__end = int(float(t) / self._dt)

    @check_units(dt=second)
    def set_dt(self, dt):
        self._dt = float(dt)
#        self._dtby2 = self._dt/2.0

    @check_units(end=second)
    def set_end(self, end):
        self.__end = int(float(end) / self._dt)

    @check_units(start=second)
    def set_start(self, start):
        self.__start = int(float(start) / self._dt)

    # Regular clock uses integers, but lots of Brian code extracts _t and _dt
    # directly from the clock, so these should be implemented directly
    _t = property(fget=lambda self:self.__t * self._dt + self._gridoffset)
    _end = property(fget=lambda self:self.__end * self._dt + self._gridoffset)
    _start = property(fget=lambda self:self.__start * self._dt)

    # Clock object internally stores floats, but these properties
    # return quantities
    if isinstance(second, Quantity):
        t = property(fget=lambda self:Quantity.with_dimensions(self._t, second.dim), fset=set_t)
        dt = property(fget=lambda self:Quantity.with_dimensions(self._dt, second.dim), fset=set_dt)
        end = property(fget=lambda self:Quantity.with_dimensions(self._end, second.dim), fset=set_end)
        start = property(fget=lambda self:Quantity.with_dimensions(self._start, second.dim), fset=set_start)
    else:
        t = property(fget=lambda self:self._t, fset=set_t)
        dt = property(fget=lambda self:self._dt, fset=set_dt)
        end = property(fget=lambda self:self._end, fset=set_end)
        start = property(fget=lambda self:self._start, fset=set_start)

    @check_units(duration=second)
    def set_duration(self, duration):
        self.__start = self.__t
        self.__end = self.__t + int(ceil(float(duration) / self._dt))

    def get_duration(self):
        return self.end - self.t

    def still_running(self):
        return self.__t < self.__end

    epsilon = 1e-14

    def __lt__(self, other):
        selft = self._t
        othert = other._t
        if selft==othert: return self.order<other.order
#        if selft<=othert-other._dtby2:
#            return True
        if abs(selft-othert)<=self.epsilon*abs(selft):
            return self.order<other.order
        return selft<othert
    
    
class RegularClock(Clock):
    '''
    Deprecated. Now the same as :class:`Clock`. The old :class:`Clock` class
    is now :class:`FloatClock`.
    '''
    pass


class FloatClock(Clock):
    '''
    Similar to a :class:`Clock` except that it uses a float value of ``t``
    rather than an integer based underlying value. This means that over time
    the values of ``t`` can drift slightly off the grid, and sometimes
    ``t/dt`` will be slightly less than an integer value, sometimes slightly
    more. This can cause problems in cases where the computation ``int(t/dt)``
    is performed to extract an index value, as sometimes an index will be
    repeated or skipped. However, this form of clock can be used for backwards
    compatibility with versions of Brian before the new integer based clock
    was introduced, and for more flexibility than the new version allows for.
    Note also that the equality condition for this clock uses an ``epsilon``
    of ``1e-8`` rather than ``1e-14``. See :class:`Clock` for more details on
    this. For full backwards compatibility with older versions of Brian, use
    :class:`NaiveClock`.
    '''
    _t, _end, _start = 0.0, 0.0, 0.0 # set this here to override the property values defined in Clock
    @check_units(dt=second, t=second)
    def __init__(self, dt=0.1*msecond, t=0*msecond, order=0,
                 makedefaultclock=False):
        self.t = t
        self.dt = dt
        self._end = float(t)
        self.order = order
        if not exists_global_preference('defaultclock') or makedefaultclock:
            set_global_preferences(defaultclock=self)

    @check_units(t=second)
    def reinit(self, t=0 * msecond):
        self._t = float(t)

    def tick(self):
        self._t += self._dt

    @check_units(dt=second)
    def set_dt(self, dt):
        self._dt = float(dt)
#        self._dtby2 = self._dt/2.0

    @check_units(t=second)
    def set_t(self, t):
        self._t = float(t)
        #self._end = float(t)

    @check_units(end=second)
    def set_end(self, end):
        """Sets the end-point for the clock
        """
        self._end = float(end)

    @check_units(start=second)
    def set_start(self, start):
        """Sets the start-point for the clock
        """
        self._start = float(start)

    # Clock object internally stores floats, but these properties
    # return quantities
    if isinstance(second, Quantity):
        t = property(fget=lambda self:Quantity.with_dimensions(self._t, second.dim), fset=set_t)
        dt = property(fget=lambda self:Quantity.with_dimensions(self._dt, second.dim), fset=set_dt)
        end = property(fget=lambda self:Quantity.with_dimensions(self._end, second.dim), fset=set_end)
        start = property(fget=lambda self:Quantity.with_dimensions(self._start, second.dim), fset=set_start)
    else:
        t = property(fget=lambda self:self._t, fset=set_t)
        dt = property(fget=lambda self:self._dt, fset=set_dt)
        end = property(fget=lambda self:self._end, fset=set_end)
        start = property(fget=lambda self:self._start, fset=set_start)

    @check_units(duration=second)
    def set_duration(self, duration):
        """Sets the duration of the clock
        """
        self._start = self._t
        self._end = self._t + float(duration)

    def get_duration(self):
        return self.end - self.t

    def still_running(self):
        """Checks if the clock is still running
        """
        return self._t < self._end

    epsilon = 1e-8


class NaiveClock(FloatClock):
    '''
    Provided for backwards compatibility with older versions of Brian. Does not
    perform any approximate equality tests for clocks, meaning that clock
    processing sequence is undpredictable. Typically, users should use
    :class:`Clock` or :class:`FloatClock`.
    '''
    def __lt__(self, other):
        selft = self._t
        othert = other._t
        if selft==othert:
            return self.order<other.order
        return selft<othert


def guess_clock(clock=None):
    '''
    Tries to guess the clock from global and local namespaces
    from the caller.
    Selects the most local clock.
    Raises an error if several clocks coexist in the same namespace.
    If a non-None clock is passed, then it is returned (simplifies the code).
    '''
    if clock:
        return clock
    # Get variables from the stack
    (clocks, clocknames) = magic.find_instances(Clock)
    if len(clocks) > 1: # several clocks: ambiguous
        # What type of error?
        raise TypeError("Clock is ambiguous. Please specify it explicitly.")
    if len(clocks) == 1:
        return clocks[0]
    # Fall back on default clock
    if exists_global_preference('defaultclock'): return get_global_preference('defaultclock')
    # No clock found
    raise TypeError("No clock found. Please define a clock.")


class EventClock(Clock):
    '''
    Clock that is used for events.
    
    Works the same as a :class:`Clock` except that it is never guessed as a clock to
    use by :class:`NeuronGroup`, etc. These clocks can be used to make multiple clock
    simulations without causing ambiguous clock problems.
    '''
    @staticmethod
    def _track_instances(): return False


# Do not track the default clock    
class DefaultClock(Clock):
    @staticmethod
    def _track_instances(): return False
defaultclock = DefaultClock(dt=0.1 * msecond)
define_global_preference(
    'defaultclock', 'Clock(dt=0.1*msecond)',
    desc="""
         The default clock to use if none is provided or defined
         in any enclosing scope.
         """)


def define_default_clock(**kwds):
    '''
    Create a new default clock
    
    Uses the keywords of the :class:`Clock` initialiser.
    
    Sample usage::
    
        define_default_clock(dt=1*ms)
    '''
    kwds['makedefaultclock'] = True
    newdefaultclock = Clock(**kwds)


def reinit_default_clock(t=0 * msecond):
    '''
    Reinitialise the default clock (to zero or a specified time)
    '''
    get_default_clock().reinit(t)


def get_default_clock():
    '''
    Returns the default clock object.
    '''
    return get_global_preference('defaultclock')

if __name__ == '__main__':
    print id(guess_clock()), id(defaultclock)



# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
"""
Various utilities for documentation
"""

def indent_string(s, numtabs=1, spacespertab=4, split=False):
    """
    Indents a given string or list of lines
    
    split=True returns the output as a list of lines
    """
    indent = ' ' * (numtabs * spacespertab)
    if isinstance(s, str):
        indentedstring = indent + s.replace('\n', '\n' + indent)
    else:
        indentedstring = ''
        for l in s:
            indentedstring += indent + l + '\n'
    indentedstring = indentedstring.rstrip() + '\n'
    if split: return indentedstring.split('\n')
    return indentedstring

def flattened_docstring(docstr, numtabs=0, spacespertab=4, split=False):
    """
    Returns a docstring with the indentation removed according to the Python standard
    
    split=True returns the output as a list of lines
    
    Changing numtabs adds a custom indentation afterwards
    """
    if isinstance(docstr, str):
        lines = docstr.split('\n')
    else:
        lines = docstr
    if len(lines) < 2: # nothing to do
        return docstr
    flattenedstring = ''
    # Interpret multiline strings according to the Python docstring standard 
    indentlevel = min(# the smallest number of whitespace characters in the lines of the description
                      map(# the number of whitespaces at the beginning of each string in the lines of the description
                          lambda l:len(l) - len(l.lstrip()), # the number of whitespaces at the beginning of the string
                          filter(# only those lines with some text on are counted
                                 lambda l:len(l.strip())
                                 , lines[1:] # ignore the first line
                                 )))
    if lines[0].strip(): # treat first line differently (probably has nothing on it)
        flattenedstring += lines[0] + '\n'
    for l in lines[1:-1]:
        flattenedstring += l[indentlevel:] + '\n'
    if lines[-1].strip(): # treat last line differently (probably has nothing on it)
        flattenedstring += lines[-1][indentlevel:] + '\n'
    return indent_string(flattenedstring, numtabs=numtabs, spacespertab=spacespertab, split=split)

def rest_section(name, level='-', split=False):
    """
    Returns a restructuredtext section heading
    
    Looks like:
    
        name
        ----
        
    """
    s = name + '\n' + level * len(name) + '\n\n'
    if split:
        return s.split('\n')
    return s

if __name__ == '__main__':
    print rest_section('Test'),
    print flattened_docstring(flattened_docstring.__doc__),



'''
Random processes for Brian.
'''
from brian.equations import Equations
from brian.units import get_unit
from scipy.signal import lfilter
from numpy import exp, sqrt
from numpy.random import randn
from brian.stdunits import ms

__all__ = ['OrnsteinUhlenbeck', 'white_noise', 'colored_noise']

def OrnsteinUhlenbeck(x, mu, sigma, tau):
    '''
    An Ornstein-Uhlenbeck process.
    mu = mean
    sigma = standard deviation
    tau = time constant
    x = name of the variable
    Returns an Equations() object
    '''
    return Equations('dx/dt=(mu-x)*invtau+sigma*((2.*invtau)**.5)*xi : unit', \
                     x=x, mu=mu, sigma=sigma, invtau=1. / tau, unit=get_unit(mu))


def white_noise(dt, duration):
    n = int(duration/dt)
    noise = randn(n)
    return noise

def colored_noise(tau, dt, duration):
    noise = white_noise(dt, duration)
    a = [1., -exp(-dt/tau)]
    b = [1.]
    fnoise = sqrt(2*dt/tau)*lfilter(b, a, noise)
    return fnoise

if __name__ == '__main__':
    tau = 10*ms
    dt = .1*ms
    duration = 1000*ms
    noise = white_noise(dt, duration)
    cnoise = colored_noise(tau, dt, duration)
    from numpy import linspace
    t = linspace(0*ms, duration, len(noise))
    
    from pylab import acorr, show, subplot, plot, ylabel, xlim
    
    subplot(221)
    plot(t, noise)
    ylabel('white noise')
    
    subplot(222)
    acorr(noise, maxlags=200)
    xlim(-200,200)
    
    subplot(223)
    plot(t, cnoise)
    ylabel('colored noise')
    
    subplot(224)
    acorr(cnoise, maxlags=200)
    xlim(-200,200)
    
    show()


from brian import *
from nose.tools import *

def test():
    """
    Statistics module
    """
    assert total_correlation([],[]) is NaN
    assert correlogram([],[]) is NaN
    assert firing_rate([]) is NaN
    assert CV([]) is NaN
    
if __name__ == '__main__':
    test()



from brian import *
from nose.tools import *

def test():
    '''
    Equations module.
    '''
    reinit_default_clock()

    tau = 2 * ms

    eqs = Equations('''
    x=v**2 : volt**2
    y=x # a comment
    dv/dt=(y-v)/tau : volt
    z : amp''')

    # Parsing and building
    assert eqs._eq_names == ['x', 'y'] # An alias is also a static equation
    assert eqs._diffeq_names == ['v', 'z'] # A parameter is a differential equation dz/dt=0
    assert eqs._diffeq_names_nonzero == ['v']
    assert eqs._alias == {'y':'x'}
    assert eqs._units == {'t':second, 'x':volt ** 2, 'z':amp, 'y':volt ** 2, 'v': volt}
    assert eqs._string == {'y': 'x', 'x': 'v**2', 'z': '0*amp/second', 'v': '(y-v)/tau'}
    assert eqs._namespace['v']['tau'] == 2 * ms
    assert 'tau' not in eqs._namespace['x']

    # Name substitutions
    assert Equations('dx/dt=-x/(2*ms):1', x='y')._diffeq_names == ['y']

    # Explicit namespace
    eqs2 = Equations('dx/dt=-x/tau:volt', tau=1 * ms)
    assert eqs2._namespace['x']['tau'] == 1 * ms
    assert eqs2._namespace['x'] == {'tau':1 * ms, 'volt':volt}

    # Find membrane potential
    assert eqs.get_Vm() == 'v'
    assert Equations('v=x**2 : 1').get_Vm() == None # must be a differential equation
    assert Equations('dx/dt=1/(2*ms) : 1').get_Vm() == None
    assert Equations('dvm/dt=1/(2*ms) : 1').get_Vm() == 'vm'
    assert Equations('''dvm/dt=1/(2*ms) : 1
    dv/dt=1/(2*ms) : 1
    ''').get_Vm() == None # ambiguous

    # Unit checking
    eqs = Equations('dv/dt=-v : volt')
    assert_raises(DimensionMismatchError, eqs.prepare)

    # Free variables
    eqs = Equations('dv/dt=(freevar-v)/tau : volt')
    assert eqs.free_variables() == ['tau', 'freevar']

    # Equation types: stochastic, time dependent, linear, conditionally linear
    eqs = Equations('''
    dv/dt=(f(t)-v)/tau : volt
    dw/dt=-w/tau+3*tau**-.5*xi : 1''')
    assert eqs.is_stochastic()
    assert eqs.is_stochastic(var='v') is False
    assert eqs.is_stochastic(var='w')
    assert eqs.is_time_dependent()
    assert eqs.is_time_dependent(var='v')
    assert eqs.is_time_dependent(var='w') is False
    assert eqs.is_linear() is False # not linear if time-dependent

    eqs = Equations('dv/dt=-v/tau : volt')
    assert eqs.is_linear()
    eqs.prepare()
    assert eqs.is_conditionally_linear()

    eqs = Equations('''
    dv/dt=(w**3-v)/tau : 1
    dw/dt=(v**2-w)/tau : 1
    ''')
    eqs.prepare()
    assert eqs.is_conditionally_linear()

    eqs = Equations('''
    dv/dt=(w**3-v**2)/tau : 1
    dw/dt=(v**2-w)/tau : 1
    ''')
    eqs.prepare()
    assert eqs.is_conditionally_linear() is False

if __name__ == '__main__':
    test()



'''
Testing the ability to construct synapses with strings!
'''

from brian import *
import dev.ideas.synapses.synapses as syn
reload(syn)
log_level_debug()

gin = NeuronGroup(1000, model = 'dv/dt = -v/(5*ms) : 1')
gout = NeuronGroup(1000, model = 'dv/dt = -v/(5*ms) : 1')

# init synapses
print 'SPARSE SYNAPSES'
synapses = syn.Synapses(gin, gout, model = '''w : 1; z : 1''', pre = 'v += w')

synapses[:,:] = '''rand() < .03'''

print 'proportion of created synapses: ', float(len(synapses)) / (len(gin) * len(gout))


del synapses
synapses = syn.Synapses(gin, gout, model = '''w : 1; z : 1''', pre = 'v += w')

print 'MULTIPLE SYNAPSES'
synapses[:,:] = ''' (i == j)'''
# this is slow



#!/usr/bin/env python

"""
setup.py file for C++ version of circular spike container
"""

from distutils.core import setup, Extension
import os, numpy
numpy_base_dir = os.path.split(numpy.__file__)[0]
numpy_include_dir = os.path.join(numpy_base_dir, 'core/include')
# 'c:\\python25\\lib\\site-packages\\numpy\\core\\include'

ccircular_module = Extension('_ccircular',
                           sources=['ccircular_wrap.cxx',
                                    'circular.cpp',
                                    ],
                           include_dirs=[numpy_include_dir],
                           )

setup (name='ccircular',
       version='0.1',
       author="Dan Goodman",
       description="""C++ version of circular spike container""",
       ext_modules=[ccircular_module],
       py_modules=["ccircular"],
       )



# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
"""
Classes and functions for storing and using parameters
"""

__all__ = ['attribdict', 'Parameters']

from itertools import chain
from inspect import *


class attribdict(dict):
    """
    Dictionary that can be accessed via keys
    
    Note that attributes starting with _ won't be added to the dict but
    will instead be directly added to the object.
    
    Note that an attribdict cannot have new real attributes
    added to it unless they start with _, because they will
    be added to the dict instead. To add a real attribute,
    use object.__setattr__(obj,name,val).
    """
    def __init__(self, **kwds):
        super(attribdict, self).__init__(**kwds)

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError

    def __setattr__(self, name, val):
        if name in dir(self) or (len(name) and name[0] == '_'):
            dict.__setattr__(self, name, val)
            return
        self[name] = val

    def __repr__(self):
        s = 'Attributes:'
        for k, v in self.iteritems():
            s += '\n    ' + k + ' = ' + str(v)
        return s


class Parameters(attribdict):
    """
    A storage class for keeping track of parameters
    
    Example usage::
    
        p = Parameters(
            a = 5,
            b = 6,
            computed_parameters = '''
            c = a + b
            ''')
        print p.c
        p.a = 1
        print p.c
    
    The first ``print`` statement will give 11, the second gives 7.
    
    Details:
    
    Call as::
    
        p = Parameters(...)
    
    Where the ``...`` consists of a list of keyword / value pairs (like a ``dict``).
    Keywords must not start with the underscore ``_`` character. Any
    keyword that starts with ``computed_`` should be a string of valid Python statements
    that compute new values based on the given ones. Whenever a non-computed value is
    changed, the computed parameters are recomputed, in alphabetical order of their
    keyword names (so ``computed_a`` is computed before ``computed_b`` for example).
    Non-computed values can be accessed and set via ``p.x``, ``p.x=1`` for example, whereas
    computed values can only be accessed and not set. New parameters can be added
    after the :class:`Parameters` object is created, including new ``computed_*`` parameters. You
    can 'derive' a new parameters object from a given one as follows::
    
        p1 = Parameters(x=1)
        p2 = Parameters(y=2,**p1)
        print p2.x
    
    Note that changing the value of ``x`` in ``p2`` will not change the value of ``x`` in ``p1`` (this
    is a copy operation).
    """
    def __init__(self, **kwds):
        super(Parameters, self).__init__(**kwds)
        self._recompute()

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            try:
                return self._computed_values[name]
            except KeyError:
                raise AttributeError

    def __setattr__(self, name, val):
        if hasattr(self, '_computed_values') and name in self._computed_values:
            raise AttributeError('Cannot set computed value')
        attribdict.__setattr__(self, name, val)
        self._recompute()

    def _recompute(self):
        cv = dict(self)
        items = self.items()
        items.sort()
        for k, v in items:
            if k[:9] == 'computed_':
                v = '\n'.join([line.strip() for line in v.split('\n')])
                exec v in cv
        for k in cv.keys():
            if k[:1] == '_': # this is used to get rid of things like __builtins__ etc.
                cv.pop(k)
        for k in self.iterkeys():
            cv.pop(k)
        object.__setattr__(self, '_computed_values', cv)

    def ascode(self, name):
        """
        Returns a string which can be executed which gives all the parameters
        
        name is the name of the Parameters variable in the local namespace:
        
        Usage:
        
        P = Parameters(x=1)
        exec P.ascode('P')
        print x
        """
        s = ''
        allvals = dict(**self)
        allvals.update(self._computed_values)
        for k in allvals.iterkeys():
            if k[:9] != 'computed_':
                s += k + '=' + name + '.' + k + '\n'
        return s

    def get_vars(self, *vars):
        '''
        Returns a tuple of variables given their names
        
        vars can be a list of string names, or a single space separated string of names.
        '''
        vars = [v.split(' ') for v in vars]
        return tuple(getattr(self, v) for v in chain(*vars))

    def __repr__(self):
        s = 'Values:'
        for k, v in self.iteritems():
            if k[:9] != 'computed_':
                s += '\n    ' + k + ' = ' + str(v)
        if len(self._computed_values):
            s += '\n_computed values:'
            for k, v in self._computed_values.iteritems():
                s += '\n    ' + k + ' = ' + str(v)
        return s

    def __call__(self, **kwds):
        '''
        Returns a copy with specified arguments overwritten
        
        Sample usage:
        
        default_p = Parameters(x=1,y=2)
        specific_p = default_p(x=3)
        '''
        p = Parameters(**self) # pylint: disable-msg=W0621
        for k, v in kwds.iteritems():
            setattr(p, k, v)
        return p

    def __reduce__(self):
        return (_load_Parameters_from_pickle, (self.items(),))

def _load_Parameters_from_pickle(items):
    return Parameters(**dict(items))

if __name__ == "__main__":
    # turn off warning about attribute defined outside __init__
    # pylint: disable-msg=W0201
    p = Parameters(
        a=5,
        b=6,
        computed_p1='''
        c = a + b
        ''',
        computed_p2='''
        x = c**2
        ''')
    q = Parameters(
        d=100,
        computed_q='''
        e = a+d
        ''',
        **p
        )
    print p.c
    p.a = 1
    print p.c
    print p
    print
    print q
    q.a = -50
    print
    print q
    print
    try:
        q.c = 5
    except AttributeError, e:
        print e
    p.y = 6
    p.computed_p3 = '''
                    w = a*b
                    '''
    print p
    p.a = 2
    print p



from brian import *
set_global_preferences(usenewbrianhears=True)
from brian.hears import *
import time
start = time.time()
cfmin, cfmax, cfN = 20*Hz, 20*kHz, 3000
cf = erbspace(cfmin, cfmax, cfN)
sound = Sound.whitenoise(100*ms)
gfb = GammatoneFilterbank(sound, cf)
ihc = FunctionFilterbank(gfb, lambda x: 3*clip(x, 0, Inf)**(1.0/3.0))
eqs = '''
dv/dt = (I-v)/(1*ms)+0.2*xi*(2/(1*ms))**.5 : 1
I : 1
'''
G = FilterbankGroup(ihc, 'I', eqs, reset=0, threshold=1, refractory=5*ms)
M = SpikeMonitor(G)
run(sound.duration)
print time.time()-start
raster_plot(M)
show()



from rewriting import *
import re

__all__ = ['single_expr', 'single_statement',
           'c_single_expr', 'c_single_statement',
           'python_single_expr', 'python_single_statement',
           'gpu_single_expr', 'gpu_single_statement',
           ]

def single_expr(expr):
    return expr.strip()

def single_statement(expr, single_expr=single_expr):
    m = re.search(r'[^><=]=', expr)
    if m:
        return expr[:m.end()] + ' ' + single_expr(expr[m.end():])
    return expr

def c_single_expr(expr):
    return rewrite_to_c_expression(single_expr(expr.strip())).strip()

def c_single_statement(expr):
    return single_statement(expr, single_expr=c_single_expr) + ';'

python_single_expr = single_expr
python_single_statement = single_statement

gpu_single_expr = c_single_expr
gpu_single_statement = c_single_statement



'''
This module is here only to maintain backward compatibility with old code that
references brian.connection rather than brian.connections. It should be removed
at some point in the future
'''

from connections import *

import log as _log
_log.log_warn('brian', 'Module "connection" in Brian is deprecated, use "brian.connections" instead.')



'''
Testing multiprocessing fro Brian
'''
from brian import *
from multiprocessing import Pool
from time import time

A = rand(3, 3)
X1 = rand(3, 1000)
X2 = rand(3, 1000)

def f(X):
    return dot(A, X)

p = Pool(2)
t1 = time()
p.map(f, [X1, X2])
t2 = time()
print t2 - t1



from base import *
from sparsematrix import *
from connectionvector import *
from constructionmatrix import *
from connectionmatrix import *
from construction import *
from propagation_c_code import *
from scipy.sparse import issparse
# we do this at the bottom because of order of import issues
#from delayconnection import * 

__all__ = ['Connection']

class Connection(magic.InstanceTracker, ObjectContainer):
    '''
    Mechanism for propagating spikes from one group to another

    A Connection object declares that when spikes in a source
    group are generated, certain neurons in the target group
    should have a value added to specific states. See
    Tutorial 2: Connections to understand this better.
    
    With arguments:
    
    ``source``
        The group from which spikes will be propagated.
    ``target``
        The group to which spikes will be propagated.
    ``state``
        The state variable name or number that spikes will be
        propagated to in the target group.
    ``delay``
        The delay between a spike being generated at the source
        and received at the target. Depending on the type of ``delay``
        it has different effects. If ``delay`` is a scalar value, then
        the connection will be initialised with all neurons having
        that delay. For very long delays, this may raise an error. If
        ``delay=True`` then the connection will be initialised as a
        :class:`DelayConnection`, allowing heterogeneous delays (a
        different delay for each synapse). ``delay`` can also be a
        pair ``(min,max)`` or a function of one or two variables, in
        both cases it will be initialised as a :class:`DelayConnection`,
        see the documentation for that class for details. Note that in
        these cases, initialisation of delays will only have the
        intended effect if used with the ``weight`` and ``sparseness``
        arguments below.
    ``max_delay``
        If you are using a connection with heterogeneous delays, specify
        this to set the maximum allowed delay (smaller values use less
        memory). The default is 5ms.
    ``modulation``
        The state variable name from the source group that scales
        the synaptic weights (for short-term synaptic plasticity).
    ``structure``
        Data structure: ``sparse`` (default), ``dense`` or
        ``dynamic``. See below for more information on structures.
    ``weight``
        If specified, the connection matrix will be initialised with
        values specified by ``weight``, which can be any of the values
        allowed in the methods `connect*`` below.
    ``sparseness``
        If ``weight`` is specified and ``sparseness`` is not, a full
        connection is assumed, otherwise random connectivity with this
        level of sparseness is assumed.
    
    **Methods**
    
    ``connect_random(P,Q,p[,weight=1[,fixed=False[,seed=None]]])``
        Connects each neuron in ``P`` to each neuron in ``Q`` with independent
        probability ``p`` and weight ``weight`` (this is the amount that
        gets added to the target state variable). If ``fixed`` is True, then
        the number of presynaptic neurons per neuron is constant. If ``seed``
        is given, it is used as the seed to the random number generators, for
        exactly repeatable results.
    ``connect_full(P,Q[,weight=1])``
        Connect every neuron in ``P`` to every neuron in ``Q`` with the given
        weight.
    ``connect_one_to_one(P,Q)``
        If ``P`` and ``Q`` have the same number of neurons then neuron ``i``
        in ``P`` will be connected to neuron ``i`` in ``Q`` with weight 1.
    ``connect(P,Q,W)``
        You can specify a matrix of weights directly (can be in any format
        recognised by NumPy). Note that due to internal implementation details,
        passing a full matrix rather than a sparse one may slow down your code
        (because zeros will be propagated as well as nonzero values).
        **WARNING:** No unit checking is done at the moment.

    Additionally, you can directly access the matrix of weights by writing::
    
        C = Connection(P,Q)
        print C[i,j]
        C[i,j] = ...
    
    Where here ``i`` is the source neuron and ``j`` is the target neuron.
    Note: if ``C[i,j]`` should be zero, it is more efficient not to write
    ``C[i,j]=0``, if you write this then when neuron ``i`` fires all the
    targets will have the value 0 added to them rather than just the
    nonzero ones.
    **WARNING:** No unit checking is currently done if you use this method.
    Take care to set the right units.
    
    **Connection matrix structures**
    
    Brian currently features three types of connection matrix structures,
    each of which is suited for different situations. Brian has two stages
    of connection matrix. The first is the construction stage, used for
    building a weight matrix. This stage is optimised for the construction
    of matrices, with lots of features, but would be slow for runtime
    behaviour. Consequently, the second stage is the connection stage,
    used when Brian is being run. The connection stage is optimised for
    run time behaviour, but many features which are useful for construction
    are absent (e.g. the ability to add or remove synapses). Conversion
    between construction and connection stages is done by the
    ``compress()`` method of :class:`Connection` which is called
    automatically when it is used for the first time.
    
    The structures are: 
    
    ``dense``
        A dense matrix. Allows runtime modification of all values. If
        connectivity is close to being dense this is probably the most
        efficient, but in most cases it is less efficient. In addition,
        a dense connection matrix will often do the wrong thing if
        using STDP. Because a synapse will be considered to exist but
        with weight 0, STDP will be able to create new synapses where
        there were previously none. Memory requirements are ``8NM``
        bytes where ``(N,M)`` are the dimensions. (A ``double`` float
        value uses 8 bytes.)
    ``sparse``
        A sparse matrix. See :class:`SparseConnectionMatrix` for
        details on implementation. This class features very fast row
        access, and slower column access if the ``column_access=True``
        keyword is specified (making it suitable for learning
        algorithms such as STDP which require this). Memory
        requirements are 12 bytes per nonzero entry for row access
        only, or 20 bytes per nonzero entry if column access is
        specified. Synapses cannot be created or deleted at runtime
        with this class (although weights can be set to zero).
    ``dynamic``
        A sparse matrix which allows runtime insertion and removal
        of synapses. See :class:`DynamicConnectionMatrix` for
        implementation details. This class features row and column
        access. The row access is slower than for ``sparse`` so this
        class should only be used when insertion and removal of
        synapses is crucial. Memory requirements are 24 bytes per
        nonzero entry. However, note that more memory than this
        may be required because memory is allocated using a
        dynamic array which grows by doubling its size when it runs
        out. If you know the maximum number of nonzero entries you will
        have in advance, specify the ``nnzmax`` keyword to set the
        initial size of the array. 
    
    **Advanced information**
    
    The following methods are also defined and used internally, if you are
    writing your own derived connection class you need to understand what
    these do.
    
    ``propagate(spikes)``
        Action to take when source neurons with indices in ``spikes``
        fired.
    ``do_propagate()``
        The method called by the :class:`Network` ``update()`` step,
        typically just propagates the spikes obtained by calling
        the ``get_spikes`` method of the ``source`` :class:`NeuronGroup`.
    '''
    #@check_units(delay=second)
    def __init__(self, source, target, state=0, delay=0 * msecond, modulation=None,
                 structure='sparse', weight=None, sparseness=None, max_delay=5 * ms, **kwds):
        if not isinstance(delay, float):
            if delay is True:
                delay = None # this instructs us to use DelayConnection, but not initialise any delays
            self.__class__ = DelayConnection
            self.__init__(source, target, state=state, modulation=modulation, structure=structure,
                          weight=weight, sparseness=sparseness, delay=delay, max_delay=max_delay, **kwds)
            return
        self.source = source # pointer to source group
        self.target = target # pointer to target group
        if isinstance(state, str): # named state variable
            self.nstate = target.get_var_index(state)
        else:
            self.nstate = state # target state index
        if isinstance(modulation, str): # named state variable
            self._nstate_mod = source.get_var_index(modulation)
        else:
            self._nstate_mod = modulation # source state index
        if isinstance(structure, str):
            structure = construction_matrix_register[structure]
        self.W = structure((len(source), len(target)), **kwds)
        self.iscompressed = False # True if compress() has been called
        source.set_max_delay(delay)
        if not isinstance(self, DelayConnection):
            self.delay = int(delay / source.clock.dt) # Synaptic delay in time bins
        self._useaccel = get_global_preference('useweave')
        self._cpp_compiler = get_global_preference('weavecompiler')
        self._extra_compile_args = ['-O3']
        if self._cpp_compiler == 'gcc':
            self._extra_compile_args += get_global_preference('gcc_options') # ['-march=native', '-ffast-math']
        self._keyword_based_init(weight=weight, sparseness=sparseness)

    def _keyword_based_init(self, weight=None, sparseness=None, **kwds):
        # Initialisation of weights
        # TODO: check consistency of weight and sparseness
        # TODO: select dense or sparse according to sparseness
        if weight is not None or sparseness is not None:
            if weight is None:
                weight = 1.0
            if sparseness is None:
                sparseness = 1
            if issparse(weight) or isinstance(weight, ndarray):
                self.connect(W=weight)
            elif sparseness == 1:
                self.connect_full(weight=weight)
            else:
                self.connect_random(weight=weight, p=sparseness)

    def propagate(self, spikes):
        if not self.iscompressed:
            self.compress()
        if len(spikes):
            # Target state variable
            sv = self.target._S[self.nstate]
            # If specified, modulation state variable
            if self._nstate_mod is not None:
                sv_pre = self.source._S[self._nstate_mod]
            # Get the rows of the connection matrix, each row will be either a
            # DenseConnectionVector or a SparseConnectionVector.
            rows = self.W.get_rows(spikes)
            if not self._useaccel: # Pure Python version is easier to understand, but slower than C++ version below
                if isinstance(rows[0], SparseConnectionVector):
                    if self._nstate_mod is None:
                        # Rows stored as sparse vectors without modulation
                        for row in rows:
                            sv[row.ind] += row
                    else:
                        # Rows stored as sparse vectors with modulation
                        for i, row in izip(spikes, rows):
                            # note we call the numpy __mul__ directly because row is
                            # a SparseConnectionVector with different mul semantics
                            sv[row.ind] += numpy.ndarray.__mul__(row, sv_pre[i])
                else:
                    if self._nstate_mod is None:
                        # Rows stored as dense vectors without modulation
                        for row in rows:
                            sv += row
                    else:
                        # Rows stored as dense vectors with modulation
                        for i, row in izip(spikes, rows):
                            sv += numpy.ndarray.__mul__(row, sv_pre[i])
            else: # C++ accelerated code, does the same as the code above but faster and less pretty
                nspikes = len(spikes)
                if isinstance(rows[0], SparseConnectionVector):
                    rowinds = [r.ind for r in rows]
                    datas = rows
                    if self._nstate_mod is None:
                        code = propagate_weave_code_sparse
                        codevars = propagate_weave_code_sparse_vars
                    else:
                        code = propagate_weave_code_sparse_modulation
                        codevars = propagate_weave_code_sparse_modulation_vars
                else:
                    if not isinstance(spikes, numpy.ndarray):
                        spikes = array(spikes, dtype=int)
                    N = len(sv)
                    if self._nstate_mod is None:
                        code = propagate_weave_code_dense
                        codevars = propagate_weave_code_dense_vars
                    else:
                        code = propagate_weave_code_dense_modulation
                        codevars = propagate_weave_code_dense_modulation_vars
                weave.inline(code, codevars,
                             compiler=self._cpp_compiler,
                             #type_converters=weave.converters.blitz,
                             extra_compile_args=self._extra_compile_args)

    def compress(self):
        if not self.iscompressed:
            self.W = self.W.connection_matrix()
            self.iscompressed = True

    def reinit(self):
        '''
        Resets the variables.
        '''
        pass

    def do_propagate(self):
        self.propagate(self.source.get_spikes(self.delay))

    def origin(self, P, Q):
        '''
        Returns the starting coordinate of the given groups in
        the connection matrix W.
        '''
        return (P._origin - self.source._origin, Q._origin - self.target._origin)

    # TODO: rewrite all the connection functions to work row by row for memory and time efficiency 

    # TODO: change this
    def connect(self, source=None, target=None, W=None):
        '''
        Connects (sub)groups P and Q with the weight matrix W (any type).
        Internally: inserts W as a submatrix.
        TODO: checks if the submatrix has already been specified.
        '''
        P = source or self.source
        Q = target or self.target
        i0, j0 = self.origin(P, Q)
        self.W[i0:i0 + len(P), j0:j0 + len(Q)] = W

    def connect_random(self, source=None, target=None, p=1., weight=1., fixed=False, seed=None, sparseness=None):
        '''
        Connects the neurons in group P to neurons in group Q with probability p,
        with given weight (default 1).
        The weight can be a quantity or a function of i (in P) and j (in Q).
        If ``fixed`` is True, then the number of presynaptic neurons per neuron is constant.
        '''
        P = source or self.source
        Q = target or self.target
        if sparseness is not None: p = sparseness # synonym
        if seed is not None:
            numpy.random.seed(seed) # numpy's random number seed
            pyrandom.seed(seed) # Python's random number seed
        if fixed:
            random_matrix_function = random_matrix_fixed_column
        else:
            random_matrix_function = random_matrix

        if callable(weight):
            # Check units
            try:
                if weight.func_code.co_argcount == 2:
                    weight(0, 0) + Q._S0[self.nstate]
                else:
                    weight() + Q._S0[self.nstate]
            except DimensionMismatchError, inst:
                raise DimensionMismatchError("Incorrects unit for the synaptic weights.", *inst._dims)
            self.connect(P, Q, random_matrix_function(len(P), len(Q), p, value=weight))
        else:
            # Check units
            try:
                weight + Q._S0[self.nstate]
            except DimensionMismatchError, inst:
                raise DimensionMismatchError("Incorrects unit for the synaptic weights.", *inst._dims)
            self.connect(P, Q, random_matrix_function(len(P), len(Q), p, value=float(weight)))

    def connect_full(self, source=None, target=None, weight=1.):
        '''
        Connects the neurons in group P to all neurons in group Q,
        with given weight (default 1).
        The weight can be a quantity or a function of i (in P) and j (in Q).
        '''
        P = source or self.source
        Q = target or self.target
        # TODO: check units
        if callable(weight):
            # Check units
            try:
                weight(0, 0) + Q._S0[self.nstate]
            except DimensionMismatchError, inst:
                raise DimensionMismatchError("Incorrects unit for the synaptic weights.", *inst._dims)
            W = zeros((len(P), len(Q)))
            try:
                x = weight(0, 1. * arange(0, len(Q)))
                failed = False
                if array(x).size != len(Q):
                    failed = True
            except:
                failed = True
            if failed: # vector-based not possible
                log_debug('connections', 'Cannot build the connection matrix by rows')
                for i in range(len(P)):
                    for j in range(len(Q)):
                        w = float(weight(i, j))
                        #if not is_within_absolute_tolerance(w,0.,effective_zero): # for sparse matrices
                        W[i, j] = w
            else:
                for i in range(len(P)): # build W row by row
                    #Below: for sparse matrices (?)
                    #w = weight(i,1.*arange(0,len(Q)))
                    #I = (abs(w)>effective_zero).nonzero()[0]
                    #print w, I, w[I]
                    #W[i,I] = w[I]
                    W[i, :] = weight(i, 1. * arange(0, len(Q)))
            self.connect(P, Q, W)
        else:
            try:
                weight + Q._S0[self.nstate]
            except DimensionMismatchError, inst:
                raise DimensionMismatchError("Incorrect unit for the synaptic weights.", *inst._dims)
            self.connect(P, Q, float(weight) * ones((len(P), len(Q))))

    def connect_one_to_one(self, source=None, target=None, weight=1):
        '''
        Connects source[i] to target[i] with weights 1 (or weight).
        '''
        P = source or self.source
        Q = target or self.target
        if (len(P) != len(Q)):
            raise AttributeError, 'The connected (sub)groups must have the same size.'
        # TODO: unit checking
        self.connect(P, Q, float(weight) * eye_lil_matrix(len(P)))

    def __getitem__(self, i):
        return self.W.__getitem__(i)

    def __setitem__(self, i, x):
        # TODO: unit checking
        self.W.__setitem__(i, x)

from delayconnection import *



'''
Random network with external noise and transmission delays
'''
from brian import *
tau = 10 * ms
sigma = 5 * mV
eqs = 'dv/dt = -v/tau+sigma*xi/tau**.5 : volt'
P = NeuronGroup(4000, model=eqs, threshold=10 * mV, reset=0 * mV, \
              refractory=5 * ms)
P.v = -60 * mV
Pe = P.subgroup(3200)
Pi = P.subgroup(800)
C = Connection(P, P, 'v', delay=2 * ms)
C.connect_random(Pe, P, 0.05, weight=.7 * mV)
C.connect_random(Pi, P, 0.05, weight= -2.8 * mV)
M = SpikeMonitor(P, True)
run(1 * second)
print 'Mean rate =', M.nspikes / 4000. / second
raster_plot(M)
show()



'''
Example of the use of the class :class:`~brian.hears.IIRFilterbank` available in
the library.  In this example, a white noise is filtered by a bank of chebyshev
bandpass filters and lowpass filters which are different for every channels.
The centre frequencies of  the filters are linearly taken between 100kHz and
1000kHz and its bandwidth or cutoff frequency increases linearly with frequency.
'''
from brian import *
from brian.hears import *

sound = whitenoise(100*ms).ramp()
sound.level = 50*dB

### example of a bank of bandpass filter ################
nchannels = 50
center_frequencies = linspace(200*Hz, 1000*Hz, nchannels)  #center frequencies 
bw = linspace(50*Hz, 300*Hz, nchannels)  #bandwidth of the filters
# The maximum loss in the passband in dB. Can be a scalar or an array of length
# nchannels
gpass = 1.*dB
# The minimum attenuation in the stopband in dB. Can be a scalar or an array
# of length nchannels
gstop = 10.*dB
#arrays of shape (2 x nchannels) defining the passband frequencies (Hz)
passband = vstack((center_frequencies-bw/2, center_frequencies+bw/2))
#arrays of shape (2 x nchannels) defining the stopband frequencies (Hz)
stopband = vstack((center_frequencies-1.1*bw, center_frequencies+1.1*bw))

filterbank = IIRFilterbank(sound, nchannels, passband, stopband, gpass, gstop,
                           'bandstop', 'cheby1')
filterbank_mon = filterbank.process()

figure()
subplot(211)
imshow(flipud(filterbank_mon.T), aspect='auto')    

#### example of a bank of lowpass filter ################
nchannels = 50
cutoff_frequencies = linspace(100*Hz, 1000*Hz, nchannels)
#bandwidth of the transition region between the en of the pass band and the
#begin of the stop band 
width_transition = linspace(50*Hz, 300*Hz, nchannels)
# The maximum loss in the passband in dB. Can be a scalar or an array of length
# nchannels
gpass = 1*dB
# The minimum attenuation in the stopband in dB. Can be a scalar or an array of
# length nchannels
gstop = 10*dB
passband = cutoff_frequencies-width_transition/2
stopband = cutoff_frequencies+width_transition/2

filterbank = IIRFilterbank(sound, nchannels, passband, stopband, gpass, gstop,
                           'low','cheby1')
filterbank_mon=filterbank.process()

subplot(212)
imshow(flipud(filterbank_mon.T), aspect='auto')    
show()



from brian import *
from ..sounds import Sound
from ..filtering import FIRFilterbank
from copy import copy

__all__ = ['HRTF', 'HRTFSet', 'HRTFDatabase',
           'make_coordinates']

class HRTF(object):
    '''
    Head related transfer function.
    
    **Attributes**

    ``impulse_response``
        The pair of impulse responses (as stereo :class:`Sound` objects)
    ``fir``
        The impulse responses in a format suitable for using with
        :class:`FIRFilterbank` (the transpose of ``impulse_response``).
    ``left``, ``right``
        The two HRTFs (mono :class:`Sound` objects)
    ``samplerate``
        The sample rate of the HRTFs.
        
    **Methods**
    
    .. automethod:: apply
    .. automethod:: filterbank
    
    You can get the number of samples in the impulse response with ``len(hrtf)``.        
    '''
    def __init__(self, hrir_l, hrir_r=None):
        if hrir_r is None:
            hrir = hrir_l
        else:
            hrir = Sound((hrir_l, hrir_r), samplerate=hrir_l.samplerate)
        self.samplerate = hrir.samplerate
        self.impulse_response = hrir
        self.left = hrir.left
        self.right = hrir.right

    def apply(self, sound):
        '''
        Returns a stereo :class:`Sound` object formed by applying the pair of
        HRTFs to the mono ``sound`` input. Equivalently, you can write
        ``hrtf(sound)`` for ``hrtf`` an :class:`HRTF` object.
        '''
        # Note we use an FFT based method for applying HRTFs that is
        # mathematically equivalent to using convolution (accurate to 1e-15
        # in practice) and around 100x faster.
        if not sound.nchannels==1:
            raise ValueError('HRTF can only be applied to mono sounds')
        if len(unique(array([self.samplerate, sound.samplerate], dtype=int)))>1:
            raise ValueError('HRTF and sound samplerates do not match.')
        sound = asarray(sound).flatten()
        # Pad left/right/sound with zeros of length max(impulse response length)
        # at the beginning, and at the end so that they are all the same length
        # which should be a power of 2 for efficiency. The reason to pad at
        # the beginning is that the first output samples are not guaranteed to
        # be equal because of the delays in the impulse response, but they
        # exactly equalise after the length of the impulse response, so we just
        # zero pad. The reason for padding at the end is so that for the FFT we
        # can just multiply the arrays, which should have the same shape.
        left = asarray(self.left).flatten()
        right = asarray(self.right).flatten()
        ir_nmax = max(len(left), len(right))
        nmax = max(ir_nmax, len(sound))+ir_nmax
        nmax = 2**int(ceil(log2(nmax)))
        leftpad = hstack((left, zeros(nmax-len(left))))
        rightpad = hstack((right, zeros(nmax-len(right))))
        soundpad = hstack((zeros(ir_nmax), sound, zeros(nmax-ir_nmax-len(sound))))
        # Compute FFTs, multiply and compute IFFT
        left_fft = fft(leftpad, n=nmax)
        right_fft = fft(rightpad, n=nmax)
        sound_fft = fft(soundpad, n=nmax)
        left_sound_fft = left_fft*sound_fft
        right_sound_fft = right_fft*sound_fft
        left_sound = ifft(left_sound_fft).real
        right_sound = ifft(right_sound_fft).real
        # finally, we take only the unpadded parts of these
        left_sound = left_sound[ir_nmax:ir_nmax+len(sound)]
        right_sound = right_sound[ir_nmax:ir_nmax+len(sound)]
        return Sound((left_sound, right_sound), samplerate=self.samplerate)        
    __call__ = apply

    def get_fir(self):
        return array(self.impulse_response.T, copy=True)
    fir = property(fget=get_fir)

    def filterbank(self, source, **kwds):
        '''
        Returns an :class:`FIRFilterbank` object that can be used to apply
        the HRTF as part of a chain of filterbanks.
        '''
        return FIRFilterbank(source, self.fir, **kwds)
    
    def __len__(self):
        return self.impulse_response.shape[0]

def make_coordinates(**kwds):
    '''
    Creates a numpy record array from the keywords passed to the function.
    Each keyword/value pair should be the name of the coordinate the array of
    values of that coordinate for each location.
    Returns a numpy record array. For example::
    
        coords = make_coordinates(azimuth=[0, 30, 60, 0, 30, 60],
                                  elevation=[0, 0, 0, 30, 30, 30])
        print coords['azimuth']
    '''
    dtype = [(name, float) for name in kwds.keys()]
    n = len(kwds.values()[0])
    x = zeros(n, dtype=dtype)
    for name, values in kwds.items():
        x[name] = values
    return x

class HRTFSet(object):
    '''
    A collection of HRTFs, typically for a single individual.
    
    Normally this object is created automatically by an :class:`HRTFDatabase`.
        
    **Attributes**
    
    ``hrtf``
        A list of ``HRTF`` objects for each index.
    ``num_indices``
        The number of HRTF locations. You can also use ``len(hrtfset)``.
    ``num_samples``
        The sample length of each HRTF.
    ``fir_serial``, ``fir_interleaved``
        The impulse responses in a format suitable for using with
        :class:`FIRFilterbank`, in serial (LLLLL...RRRRR....) or interleaved
        (LRLRLR...).
    
    **Methods**
    
    .. automethod:: subset
    .. automethod:: filterbank
    
    You can access an HRTF by index via ``hrtfset[index]``, or
    by its coordinates via ``hrtfset(coord1=val1, coord2=val2)``.
    
    **Initialisation**
    
    ``data``
        An array of shape (2, num_indices, num_samples) where data[0,:,:] is
        the left ear and data[1,:,:] is the right ear, num_indices is the number
        of HRTFs for each ear, and num_samples is the length of the HRTF.
    ``samplerate``
        The sample rate for the HRTFs (should have units of Hz).
    ``coordinates``
        A record array of length ``num_indices`` giving the coordinates of each
        HRTF. You can use :func:`make_coordinates` to help with this.
    '''
    def __init__(self, data, samplerate, coordinates):
        self.data = data
        self.samplerate = samplerate
        self.coordinates = coordinates
        self.hrtf = []
        for i in xrange(self.num_indices):
            l = Sound(self.data[0, i, :], samplerate=self.samplerate)
            r = Sound(self.data[1, i, :], samplerate=self.samplerate)
            self.hrtf.append(HRTF(l, r))
            
    def __getitem__(self, key):
        return self.hrtf[key]
    
    def __call__(self, **kwds):
        I = ones(self.num_indices, dtype=bool)
        for key, value in kwds.items():
            I = logical_and(I, abs(self.coordinates[key]-value)<1e-10)
        indices = I.nonzero()[0]
        if len(indices)==0:
            raise IndexError('No HRTF exists with those coordinates')
        if len(indices)>1:
            raise IndexError('More than one HRTF exists with those coordinates')
        return self.hrtf[indices[0]]

    def subset(self, condition):
        '''
        Generates the subset of the set of HRTFs whose coordinates satisfy
        the ``condition``. This should be one of: a boolean array of
        length the number of HRTFs in the set, with values
        of True/False to indicate if the corresponding HRTF should be included
        or not; an integer array with the indices of the HRTFs to keep; or a
        function whose argument names are
        names of the parameters of the coordinate system, e.g.
        ``condition=lambda azim:azim<pi/2``.
        '''
        if callable(condition):
            ns = dict((name, self.coordinates[name]) for name in condition.func_code.co_varnames)
            try:
                I = condition(**ns)
                I = I.nonzero()[0]
            except:
                I = False
            if isinstance(I, bool): # vector-based calculation doesn't work
                n = len(ns[condition.func_code.co_varnames[0]])
                I = array([condition(**dict((name, ns[name][j]) for name in condition.func_code.co_varnames)) for j in range(n)])
                I = I.nonzero()[0]
        else:
            if condition.dtype==bool:
                I = condition.nonzero()[0]
            else:
                I = condition
        hrtf = [self.hrtf[i] for i in I]
        coords = self.coordinates[I]
        data = self.data[:, I, :]
        obj = copy(self)
        obj.hrtf = hrtf
        obj.coordinates = coords
        obj.data = data
        return obj
    
    def __len__(self):
        return self.num_indices
    
    @property
    def num_indices(self):
        return self.data.shape[1]
    
    @property
    def num_samples(self):
        return self.data.shape[2]
    
    @property
    def fir_serial(self):
        return reshape(self.data, (self.num_indices*2, self.num_samples))
    
    @property
    def fir_interleaved(self):
        fir = empty((self.num_indices*2, self.num_samples))
        fir[::2, :] = self.data[0, :, :]
        fir[1::2, :] = self.data[1, :, :]
        return fir
    
    def filterbank(self, source, interleaved=False, **kwds):
        '''
        Returns an :class:`FIRFilterbank` object which applies all of the HRTFs
        in the set. If ``interleaved=False`` then
        the channels are arranged in the order LLLL...RRRR..., otherwise they
        are arranged in the order LRLRLR....
        '''
        if interleaved:
            fir = self.fir_interleaved
        else:
            fir = self.fir_serial
        return FIRFilterbank(source, fir, **kwds)


class HRTFDatabase(object):
    '''
    Base class for databases of HRTFs
    
    Should have an attribute 'subjects' giving a list of available subjects,
    and a method ``load_subject(subject)`` which returns an ``HRTFSet`` for that
    subject.
    
    The initialiser should take (optional) keywords:
    
    ``samplerate``
        The intended samplerate (resampling will be used if it is wrong). If
        left unset, the natural samplerate of the data set will be used.    
    '''
    def __init__(self, samplerate=None):
        raise NotImplementedError

    def load_subject(self, subject):
        raise NotImplementedError



# TODO: add gains to this model, to make it equivalent to original model
from brian import *
set_global_preferences(usenewbrianhears=True)
from brian.hears import *
from plot_count import ircam_plot_count
import time

hrtfdb = IRCAM_LISTEN(r'D:\HRTF\IRCAM')
subject = 1002
hrtfset = hrtfdb.load_subject(subject)
index = randint(hrtfset.num_indices)
cfmin, cfmax, cfN = 150*Hz, 5*kHz, 40
sound = Sound.whitenoise(500*ms)

hrtf = hrtfset.hrtf[index]
cf = erbspace(cfmin, cfmax, cfN)
num_indices = hrtfset.num_indices

hrtf_fb = hrtf.filterbank(sound)
hrtfset_fb = hrtfset.filterbank(
        RestructureFilterbank(hrtf_fb, 
                indexmapping=repeat([1, 0], hrtfset.num_indices)), minimum_buffer_size=1024)
gfb = GammatoneFilterbank(
            RestructureFilterbank(hrtfset_fb, cfN),
            tile(cf, hrtfset_fb.nchannels))
cochlea = FunctionFilterbank(gfb, lambda x:15*clip(x, 0, Inf)**(1.0/3.0))

eqs = '''
dV/dt = (I-V)/(1*ms)+0.1*xi/(0.5*ms)**.5 : 1
I : 1
'''
G = FilterbankGroup(cochlea, 'I', eqs, reset=0, threshold=1, refractory=5*ms)
cd = NeuronGroup(num_indices*cfN, eqs, reset=0, threshold=1, clock=G.clock)

C = Connection(G, cd, 'V')
for i in xrange(num_indices*cfN):
    C[i, i] = 0.5
    C[i+num_indices*cfN, i] = 0.5

M = SpikeMonitor(G, record=sound.duration<50*ms)
Mcd = SpikeMonitor(cd, record=sound.duration<50*ms)
counter = SpikeCounter(cd)

start = time.time()
run(sound.duration, report='stderr')
print 'Time taken:', time.time()-start

count = counter.count
count.shape = (num_indices, cfN)
count = count.T
count = reshape(count, (count.size,))

print 'Input spike rate:', float(M.nspikes)/(len(G)*G.clock.t)
print 'Coincidence spike rate:', float(Mcd.nspikes)/(len(cd)*G.clock.t)

if sound.duration<50*ms:
    subplot(221)
    raster_plot(M)
    subplot(223)
    raster_plot(Mcd)
    subplot(122)
ircam_plot_count(hrtfset, count, index=index)
show()



from brian import *
if False:
    tau = 1 * second
    eqs = '''
    dy/dt = v/tau : 1
    dv/dt = -y/tau : 1
    dx/dt = exp(y)/tau : 1
    dw/dt = z/tau : 1 # w should be the same as x
    dz/dt = v*z/tau : 1
    '''
    G = NeuronGroup(1, eqs)
    M = MultiStateMonitor(G, record=True, when='start')
    G.y = 0
    G.v = 1
    G.x = 0
    G.w = G.x
    G.z = exp(G.y)
    run(1 * second)
    M.plot()
    legend()
    show()
if True:
    tau = 1 * second
    eqs = '''
    dy/dt=exp(y)/tau:1
    dx/dt=u/tau:1
    du/dt=u*u/tau:1
    '''
    G = NeuronGroup(1, eqs)
    M = MultiStateMonitor(G, record=True, when='start')
    G.y = 1
    G.x = G.y
    G.u = exp(G.y)
    run(exp(-1) * 0.99 * second)
    z = -log(-M.times + exp(-1))
    subplot(211)
    plot(M.times, M['x', 0], label='dx/dt=u, du/dt=u*u')
    plot(M.times, M['y', 0], label='dy/dt=exp(y)')
    plot(M.times, z, label='analytic soln') # analytic solution
    legend(loc='upper left')
    subplot(212)
    plot(M.times, abs(M['x', 0] - z), label='abs err in x')
    plot(M.times, abs(M['y', 0] - z), label='abs err in y')
    legend(loc='upper left')
    print 'max err in x', max(abs(M['x', 0] - z))
    print 'max err in y', max(abs(M['y', 0] - z))
    show()



import pycuda.autoinit as autoinit
import pycuda.driver as drv
from pycuda.gpuarray import GPUArray
from pycuda import gpuarray
import bisect
import numpy, pylab, time

N = 1000000
blocksize = 512
duration = 10000

N = int(numpy.ceil(1. * N / blocksize) * blocksize)

if drv.get_version() == (2, 0, 0): # cuda version
    precision = 'float'
elif drv.get_version() > (2, 0, 0):
    precision = 'double'
else:
    raise Exception, "CUDA 2.0 required"

if precision == 'double':
    mydtype = numpy.float64
else:
    mydtype = numpy.float32

block = (blocksize, 1, 1)
grid = (N / blocksize, 1)

mod = drv.SourceModule("""
/*__global__ void stateupdate(SCALAR *V_arr, SCALAR *ge_arr, SCALAR *gi_arr)
{
    int i = blockIdx.x * blockDim.x + threadIdx.x;
    SCALAR V = V_arr[i];
    SCALAR ge = ge_arr[i];
    SCALAR gi = gi_arr[i]; 
//    SCALAR V__tmp = (ge+gi-(V+0.049))/0.02;
//    SCALAR ge__tmp = -ge/0.005;
//    SCALAR gi__tmp = -gi/0.01;
    SCALAR V__tmp = (ge+gi-(V+0.049))*50;
    SCALAR ge__tmp = -ge*200;
    SCALAR gi__tmp = -gi*100;
    V_arr[i] = V+0.0001*V__tmp;
    ge_arr[i] = ge+0.0001*ge__tmp;
    gi_arr[i] = gi+0.0001*gi__tmp;
}*/

__global__ void stateupdate(SCALAR *V_arr, SCALAR *ge_arr, SCALAR *gi_arr)
{
    int i = blockIdx.x * blockDim.x + threadIdx.x;
    SCALAR V = V_arr[i];
    SCALAR ge = ge_arr[i];
    SCALAR gi = gi_arr[i];
    V_arr[i] = V+1;
    ge_arr[i] = ge+1;
    gi_arr[i] = gi+1;
}

__global__ void stateupdate_noglobal(SCALAR *V_arr, SCALAR *ge_arr, SCALAR *gi_arr)
{
    int i = blockIdx.x * blockDim.x + threadIdx.x;
    SCALAR V = 0.;//V_arr[i];
    SCALAR ge = 0.;//ge_arr[i];
    SCALAR gi = 0.;//gi_arr[i]; 
    SCALAR V__tmp = (ge+gi-(V+0.049))/0.02;
    SCALAR ge__tmp = -ge/0.005;
    SCALAR gi__tmp = -gi/0.01;
    V = V+0.0001*V__tmp;
    ge = ge+0.0001*ge__tmp;
    gi = gi+0.0001*gi__tmp;
}
""".replace('SCALAR', precision))
stateupdate = mod.get_function("stateupdate")
stateupdate_noglobal = mod.get_function("stateupdate_noglobal")

V = gpuarray.to_gpu(numpy.array(numpy.random.rand(N) * 0.01 - 0.06, dtype=mydtype))
ge = gpuarray.to_gpu(numpy.zeros(N, dtype=mydtype))
gi = gpuarray.to_gpu(numpy.zeros(N, dtype=mydtype))

stateupdate.prepare(('i', 'i', 'i'), block)
stateupdate_args = (grid, int(V.gpudata), int(ge.gpudata), int(gi.gpudata))
stateupdate.prepared_call(*stateupdate_args)
#stateupdate_noglobal.prepared_call(*stateupdate_args)

def run_sim(stateupdate=stateupdate):
    for t in xrange(duration):
        stateupdate.launch_grid(*grid)

start = time.time()
run_sim()
duration_stateupdate = time.time() - start

start = time.time()
run_sim(stateupdate_noglobal)
duration_stateupdate_noglobal = time.time() - start

data_copied_gb = N * mydtype().nbytes * 3 * duration * 2. / 1024. ** 3
transfer_rate_gb_sec = data_copied_gb / (duration_stateupdate - duration_stateupdate_noglobal)

print 'N:', N, 'blocksize:', blocksize, 'numsteps:', duration
print 'GPU time:', duration_stateupdate
print 'GPU time (no global memory access):', duration_stateupdate_noglobal
print 'Data transferred (GB):', data_copied_gb
print 'Data transfer rate (GB/sec):', transfer_rate_gb_sec



# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
"""Magic function tools

Use these functions to automagically find objects of a particular class. The way it works is that
whenever a new object is created from a class derived from InstanceTracker, it is stored along
with the 'frame' it was called from (loosely speaking, the function where the object is defined).
When you call one of the functions below, it picks out all the objects of the required type in
a frame a specified number of levels before the current one (e.g. in the frame of the calling
function). 

Functions
---------

get_instances(instancetype, level=1, all=False)
    This function finds all instances at the given level in the sequence of
    calling frames.
    If all is True then levels will not be used and all instances will be found.
    
find_instances(instancetype, startlevel=1, all=False)
    This function searches the frames starting from the given startlevel until it finds at least
    one object of the required type, at which point it will return all objects of that type from
    that level
    If all is True then levels will not be used and all instances will be found.

find_all_instances(instancetype,startlevel=1, all=False):
    This function searches the frames starting from the given startlevel, and returns all objects of
    the required type from all levels. Noe that includeglobals is set to False by default so as not
    to pick up multiple copies of objects
    If all is True then levels will not be used and all instances will be found.

Variables:

instancetype
    A class (must be derived from InstanceTracker)
level
    The level in the sequence of calling frames. So, for a function f, calling with level=0 will
    find variables defined in that function f, whereas calling with level=1 will find variables
    defined in the function which called f. The latter is the default value because magic
    functions are usually used within Brian functions to find variables defined by the user.

Return values:

All the functions return a tuple (objects, names) where objects is the list of matching objects,
and names is a list of strings giving the objects' names if they are defined. At the moment, the
only name returned is the id of the object.

Notes:

These functions return each object at most once.

Classes
-------

InstanceTracker
    Derive your class from this one to automagically keep track of instances of it. If you
    want a subclass of a tracked class not to be tracked, define the method _track_instances
    to return False.
      
"""

__docformat__ = "restructuredtext en"

from weakref import *
from inspect import *
from globalprefs import *

__all__ = [ 'get_instances', 'find_instances', 'find_all_instances', 'magic_register', 'magic_return', 'InstanceTracker' ]


class ExtendedRef(ref):
    """A weak reference which also defines an optional id
    """
    def __init__(self, ob, callback=None, **annotations):
        super(ExtendedRef, self).__init__(ob, callback)
        self.__id = 0

    def set_i_d(self, id):
        self.__id = id

    def get_i_d(self):
        return self.__id


class WeakSet(set):
    """A set of extended references
    
    Removes references from the set when they are destroyed."""
    def add(self, value, id=0):
        wr = ExtendedRef(value, self.remove)
        wr.set_i_d(id)
        set.add(self, wr)

    def set_i_d(self, value, id):
        for _ in self:
            if _() is value:
                _.set_i_d(id)
                return

    def get_i_d(self, value):
        for _ in self:
            if _() is value:
                return _.get_i_d()

    def get(self, id=None):
        if id is None:
            return [ _() for _ in self if _.get_i_d() != -1 ]
        else:
            return [ _() for _ in self if _.get_i_d() == id]


class InstanceFollower(object):
    """Keep track of all instances of classes derived from InstanceTracker
    
    The variable __instancesets__ is a dictionary with keys which are class
    objects, and values which are WeakSets, so __instanceset__[cls] is a
    weak set tracking all of the instances of class cls (or a subclass).
    """
    __instancesets__ = {}
    def add(self, value, id=0):
        for cls in value.__class__.__mro__: # MRO is the Method Resolution Order which contains all the superclasses of a class
            if cls not in self.__instancesets__:
                self.__instancesets__[cls] = WeakSet()
            self.__instancesets__[cls].add(value, id)

    def set_i_d(self, value, id):
        for cls in value.__class__.__mro__: # MRO is the Method Resolution Order which contains all the superclasses of a class
            if cls in self.__instancesets__:
                self.__instancesets__[cls].set_i_d(value, id)

    def get_i_d(self, value):
        for cls in value.__class__.__mro__:
            if cls in self.__instancesets__:
                return self.__instancesets__[cls].get_i_d(value)

    def get(self, cls, id=None):
        if not cls in self.__instancesets__: return []
        return self.__instancesets__[cls].get(id)


class InstanceTracker(object):
    """Base class for all classes whose instances are to be tracked
    
    Derive your class from this one to automagically keep track of instances of it. If you
    want a subclass of a tracked class not to be tracked, define the method _track_instances
    to return False.
    """
    __instancefollower__ = InstanceFollower() # static property of all objects of class derived from InstanceTracker
    @staticmethod
    def _track_instances():
        return True

    def set_instance_id(self, idvalue=None, level=1):
        if idvalue is None:
            idvalue = id(getouterframes(currentframe())[level + 1][0])
        self.__instancefollower__.set_i_d(self, idvalue)

    def get_instance_id(self):
        return self.__instancefollower__.get_i_d(self)

    def __new__(typ, *args, **kw):
        obj = object.__new__(typ)#, *args, **kw)
        outer_frame = id(getouterframes(currentframe())[1][0]) # the id is the id of the calling frame
        if obj._track_instances():
            obj.__instancefollower__.add(obj, outer_frame)
        return obj

def magic_register(*args, **kwds):
    '''Declare that a magically tracked object should be put in a particular frame
    
    **Standard usage**
    
    If ``A`` is a tracked class (derived from :class:`InstanceTracker`), then the following wouldn't
    work::
    
        def f():
            x = A('x')
            return x
        objs = f()
        print get_instances(A,0)[0]
    
    Instead you write::
    
        def f():
            x = A('x')
            magic_register(x)
            return x    
        objs = f()
        print get_instances(A,0)[0]
    
    **Definition**
    
    Call as::
    
        magic_register(...[,level=1])
    
    The ``...`` can be any sequence of tracked objects or containers of tracked objects,
    and each tracked object will have its instance id (the execution frame in which it was
    created) set to that of its parent (or to its parent at the given level). This is
    equivalent to calling::
    
        x.set_instance_id(level=level)
    
    For each object ``x`` passed to :func:`magic_register`.
    '''
    level = kwds.get('level', 1)
    for x in args:
        if isinstance(x, InstanceTracker):
            x.set_instance_id(level=level + 1)
        else:
            magic_register(*x, **{'level':level + 1})


def magic_return(f):
    '''
    Decorator to ensure that the returned object from a function is recognised by magic functions
    
    **Usage example:** ::

        @magic_return
        def f():
            return PulsePacket(50*ms, 100, 10*ms)
    
    **Explanation**
    
    Normally, code like the following wouldn't work::

        def f():
            return PulsePacket(50*ms, 100, 10*ms)
        pp = f()
        M = SpikeMonitor(pp)
        run(100*ms)
        raster_plot()
        show()
    
    The reason is that the magic function :func:`run()` only recognises objects created
    in the same execution frame that it is run from. The :func:`magic_return` decorator
    corrects this, it registers the return value of a function with the magic
    module. The following code will work as expected::

        @magic_return
        def f():
            return PulsePacket(50*ms, 100, 10*ms)
        pp = f()
        M = SpikeMonitor(pp)
        run(100*ms)
        raster_plot()
        show()
    
    **Technical details**
    
    The :func:`magic_return` function uses :func:`magic_register` with the default ``level=1``
    on just the object returned by a function. See details for :func:`magic_register`.
    '''
    def new_f(*args, **kwds):
        obj = f(*args, **kwds)
        magic_register(obj)
        return obj
    new_f.__name__ = f.__name__
    new_f.__doc__ = f.__doc__
    return new_f

def get_instances(instancetype, level=1, all=False):
    """Find all instances of a given class at a given level in the stack
    
    See documentation for module Brian.magic
    """
    try:
        instancetype.__instancefollower__
    except AttributeError:
        raise InstanceTrackerError('Cannot track instances of type ', instancetype)
    target_frame = id(getouterframes(currentframe())[level + 1][0])
    if all or not get_global_preference('magic_useframes'):
        target_frame = None
    objs = instancetype.__instancefollower__.get(instancetype, target_frame)
    return (objs, map(str, map(id, objs)))

def find_instances(instancetype, startlevel=1, all=False):
    """Find first instances of a given class in the stack
    
    See documentation for module Brian.magic
    """
    # Note that we start from startlevel+1 because startlevel means from the calling function's point of view 
    for level in range(startlevel + 1, len(getouterframes(currentframe()))):
        objs, names = get_instances(instancetype, level, all=all)
        if len(objs):
            return (objs, names)
    return ([], [])

def find_all_instances(instancetype, startlevel=1, all=False):
    """Find all instances of a given class in the stack
    
    See documentation for module Brian.magic
    """
    objs = []
    names = []
    # Note that we start from startlevel+1 because startlevel means from the calling function's point of view 
    for level in range(startlevel + 1, len(getouterframes(currentframe()))):
        newobjs, newnames = get_instances(instancetype, level, all=all)
        objs += newobjs
        names += newnames
    return (objs, names)

if __name__ == '__main__':
    print __doc__



"""
Brian on multiple processors
"""
from brian import *
from multiprocessing import Queue, Process, Pool
from time import time

eqs = '''
dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
dge/dt = -ge/(5*ms) : volt
dgi/dt = -gi/(10*ms) : volt
'''

P = NeuronGroup(4000, model=eqs,
              threshold= -50 * mV, reset= -60 * mV)
P.v = -60 * mV + 10 * mV * rand(len(P))
Pe = P.subgroup(3200)
Pi = P.subgroup(800)

Ce = Connection(Pe, P, 'ge')
Ci = Connection(Pi, P, 'gi')
Ce.connect_random(Pe, P, 0.02, weight=1.62 * mV)
Ci.connect_random(Pi, P, 0.02, weight= -9 * mV)

M = SpikeMonitor(P)

duration = 1 * second
t = 0 * second
p = Pool(2)
t1 = time()
while t < duration:
    #p=Process(NeuronGroup.update,[Pe,Pi])
    #P.reset()
    Ce.do_propagate()
    Ci.do_propagate()
    M()
    t += .1 * ms
t2 = time()
print t2 - t1
raster_plot(M)
show()



'''
Testing whether or not GCC generates code using SSE extensions, and if not,
can we make it so that it does.
'''
from brian import *
from scipy import weave
import time

dtype = float32
repeats = 100
N = 1000000
complexity = 10
x = zeros(N, dtype=dtype)
y = ones(N, dtype=dtype)

if dtype is float32:
    supp = '''
    typedef float scalar;
    typedef float vec __attribute__ ((vector_size (16)));
    '''
else:
    supp = '''
    typedef double scalar;
    typedef double vec __attribute__ ((vector_size (16)));
    '''

code = '''
for(int j=0;j<repeats;j++)
  for(int i=0;i<N;i++)
  {
    scalar &vx = x[i];
    scalar &vy = y[i];
    vx += VY;
  }
'''.replace('VY', '*'.join(['vy'] * complexity))

if dtype is float32:
    scalarsize = 4
    vecsize = 4
else:
    scalarsize = 8
    vecsize = 2
code2 = '''
int i0, i1;
i0 = ((unsigned int)x%16)/SCALARSIZE;
i1 = ((N-i0)/VECSIZE)*VECSIZE;
for(int j=0;j<repeats;j++)
{
  for(int i=0;i<i0;i++)
  {
    scalar &vx = x[i];
    scalar &vy = y[i];
    vx += VY;
  }
  for(int i=i0;i<i1;i+=VECSIZE)
  {
    vec &vx = *(vec*)(x+i);
    vec &vy = *(vec*)(y+i);
    vx += VY;
  }
  for(int i=i1;i<N;i++)
  {
    scalar &vx = x[i];
    scalar &vy = y[i];
    vx += VY;
  }
}
'''.replace('VY', '*'.join(['vy'] * complexity)).replace('VECSIZE', str(vecsize)).replace('SCALARSIZE', str(scalarsize))

def f(N, repeats, force):
    start = time.time()
    weave.inline(code, ['x', 'y', 'N', 'repeats'], compiler='gcc',
                 extra_compile_args=[
                    '-march=native', '-ffast-math',
                    '-O3',
                    ],
                 support_code=supp,
                 verbose=2 * force,
                 force=force,
                 )
    end = time.time()
    return end - start

f(0, 0, True)
print 'Time taken:', f(N, repeats, False)
print 'Correct:', (x == repeats).all()



from brian import *
import sys

log_level_warn()

if 0:
    class Clock(Clock):
        def tick(self):
            print 'Ticking:', id(self), self
            super(Clock, self).tick()

def repfunc(layer1):
    def rep(spikes):
        print 'rep:', id(layer1), layer1.clock.t, spikes
    return rep

for ii in range(0, 5):
    # Setup a new clock
    sim_clk = Clock(dt=1 * ms)
    if 0:
        print 'This sim clock:', id(sim_clk)

    # Setup up a particular sequence of spikes
    spiketimes = [(0, 1 * ms), (0, 2 * ms), (0, 3 * ms), (0, 4 * ms)]
    layer1 = SpikeGeneratorGroup(2, spiketimes, clock=sim_clk)
    if 0:
        print 'SGG clock:', id(layer1.clock)

    # Setup output recorder
    spikes1 = SpikeMonitor(layer1)
    if 0:
        spikes2 = SpikeMonitor(layer1, function=repfunc(layer1))

    if 0:
        @network_operation(clock=sim_clk)
        def rep2():
            print 'NAS:', layer1._next_allowed_spiketime

    net = MagicNetwork()
    net.prepare()
    if 1:
        print 'Groups:', map(id, net.groups)
        print 'Connections:', map(id, net.connections)
        print 'Update schedule:'
        for clk, v in net._update_schedule.items():
            print '  Clock:', id(clk)
            print '  Schedule:', v
        sys.stdout.flush()
    if 0:
        if hasattr(net, 'clocks'):
            print 'Clocks =', map(id, net.clocks)
    net.run(5 * ms)

    print 'Counted', spikes1.nspikes, 'spikes.'

    #forget(layer1)
    #clear()



# -*- coding:utf-8 -*-
"""
Sounds generated by neurons

Doesn't really work!
"""
from brian import *
from brian.hears import *
import time

eqs = '''
dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
dge/dt = -ge/(5*ms) : volt
dgi/dt = -gi/(10*ms) : volt
'''

P = NeuronGroup(4000, model=eqs,
              threshold= -50 * mV, reset= -60 * mV)
P.v = -60 * mV + 10 * mV * rand(len(P))
Pe = P.subgroup(3200)
Pi = P.subgroup(800)

Ce = Connection(Pe, P, 'ge', weight=1.62 * mV, sparseness=0.02)
Ci = Connection(Pi, P, 'gi', weight= -9 * mV, sparseness=0.02)

def play(spikes):
    if len(spikes) > 0:
        click(0.1 * ms, clip(len(spikes) * .1, 0, 1)).extend(0.1 * ms).play()
        #time.sleep(1*ms)

M = SpikeMonitor(P, function=play)
#click(1*ms,.1).play()
#whitenoise(10*ms).play()

run(1 * second)



'''
Reliability of spike timing.
See e.g. Mainen & Sejnowski (1995) for experimental results in vitro.

Here: a constant current is injected in all trials.

R. Brette
'''
from brian import *

N = 25
tau = 20 * ms
sigma = .015
eqs_neurons = '''
dx/dt=(1.1-x)/tau+sigma*(2./tau)**.5*xi:1
'''
neurons = NeuronGroup(N, model=eqs_neurons, threshold=1, reset=0, refractory=5 * ms)
spikes = SpikeMonitor(neurons)

run(500 * ms)
raster_plot(spikes)
show()



from brian import *
set_global_preferences(usenewbrianhears=True)
from brian.hears import *

hrtfdb = IRCAM_LISTEN(r'D:\HRTF\IRCAM')
subject = 1002
hrtfset = hrtfdb.load_subject(subject)
hrtf = hrtfset.hrtf[0]

x = Sound(randn(22050, 1), samplerate=44100*Hz)

fb = hrtfset.filterbank(x,
                        interleaved=True
                        )
#fb.minimum_buffer_size = 500
print fb.minimum_buffer_size
fb.buffer_init()
import time
start = time.time()
z = fb.buffer_fetch(0, 22050)
#z1 = fb.buffer_fetch(0, 250)
#z2 = fb.buffer_fetch(250, 500)
#z3 = fb.buffer_fetch(500, 750)
#z4 = fb.buffer_fetch(750, 1000)
#z = vstack((z1, z2, z3, z4))
print 'time:', time.time()-start

L = array(hrtf.apply(x).left).flatten()
l = z[:, 0]

R = array(hrtf.apply(x).right).flatten()
r = z[:, 1] # interleaved
#r = z[:, 187] # serial

print amax(abs(L-l)), amax(abs(R-r))

plot(l)
plot(r)
show()



from base import *

__all__ = ['SparseMatrix', 'sparse']

use_sparse_matrix = 'scipy_patch' # values are own, own_scipy, scipy, scipy_patch
if use_sparse_matrix == 'own':
    from ..utils import sparse_matrix as sparse
elif use_sparse_matrix == 'own_scipy':
    from ..utils import sparse # Brian's version of scipy sparse matrix library
elif use_sparse_matrix == 'scipy_patch':
    from ..utils import sparse_patch as sparse

# set this to True using the sparse library packaged with Brian, from scipy 0.7.1
if use_sparse_matrix == 'own' or use_sparse_matrix == 'own_scipy':
    oldscipy = True
else:
    oldscipy = scipy.__version__.startswith('0.6.') or scipy.__version__.startswith('0.7.1')

if use_sparse_matrix == 'own' or use_sparse_matrix == 'scipy_patch':
    SparseMatrix = sparse.lil_matrix
else:
    class SparseMatrix(sparse.lil_matrix):
        '''
        Used as the base for sparse construction matrix classes, essentially just scipy's lil_matrix.
        
        The scipy lil_matrix class allows you to specify slices in ``__setitem__`` but the
        performance is cripplingly slow. This class has a faster implementation.
        '''
        # Unfortunately we still need to implement this because although scipy 0.7.0
        # now supports X[a:b,c:d] for sparse X it is unbelievably slow (shabby code
        # on their part).
        def __setitem__(self, index, W):
            """
            Speed-up if x is a sparse matrix.
            TODO: checks (first remove the data).
            """
            try:
                i, j = index
            except (ValueError, TypeError):
                raise IndexError, "invalid index"

            if isinstance(i, slice) and isinstance(j, slice) and\
               (i.step is None) and (j.step is None) and\
               (isinstance(W, sparse.spmatrix) or isinstance(W, numpy.ndarray)):
                rows = self.rows[i]
                datas = self.data[i]
                j0 = j.start
                if isinstance(W, sparse.lil_matrix):
                    for row, data, rowW, dataW in izip(rows, datas, W.rows, W.data):
                        jj = bisect.bisect(row, j0) # Find the insertion point
                        row[jj:jj] = [j0 + k for k in rowW]
                        data[jj:jj] = dataW
                elif isinstance(W, ndarray):
                    nq = W.shape[1]
                    for row, data, rowW in izip(rows, datas, W):
                        jj = bisect.bisect(row, j0) # Find the insertion point
                        row[jj:jj] = range(j0, j0 + nq)
                        data[jj:jj] = rowW
            elif oldscipy and isinstance(i, int) and isinstance(j, (list, tuple, numpy.ndarray)):
                row = dict(izip(self.rows[i], self.data[i]))
                try:
                    row.update(dict(izip(j, W)))
                except TypeError:
                    row.update(dict(izip(j, itertools.repeat(W))))
                items = row.items()
                items.sort()
                row, data = izip(*items)
                self.rows[i] = list(row)
                self.data[i] = list(data)
            elif isinstance(i, slice) and isinstance(j, int) and isSequenceType(W):
                # This corrects a bug in scipy sparse matrix as of version 0.7.0, but
                # it is not efficient!
                for w, k in izip(W, xrange(*i.indices(self.shape[0]))):
                    sparse.lil_matrix.__setitem__(self, (k, j), w)
            else:
                sparse.lil_matrix.__setitem__(self, index, W)



from brian import *
#set_global_preferences(usecstdp=False)
#log_level_debug()

G = NeuronGroup(1, 'V:1', reset=0, threshold=1)
C = Connection(G, G, 'V',
               delay=True, max_delay=5*ms
               )

eqs_stdp = """
dA_pre/dt  = -A_pre/(10*ms)   : 1
dA_post/dt = -A_post/(10*ms) : 1
"""
stdp = STDP(C, eqs=eqs_stdp,
            pre='''
            A_pre += 1
            w += A_pre*A_post
            A_pre += 1
            ''',
            post='''
            A_post += 1
            w += A_pre
            ''', wmax=1)

G.V = 2

run(1*ms)



"""
Benchmark of a large network matching the specifications in
Morrison et al. (2005).
(Network from Brunel 2000).

Without synapses: 122s
With synapses: 220s

NEST: 1600-6400s depending on the machine

Interesting point: spike propagations are approximately as
expensive as state updates although the number of synapses is
huge. This is in fact not so surprising:

250 000 spike propagations / second
100 000 state updates / second
(and each state update involves random number generation)

This suggests that spike propagation is generally not a big issue
in practice.

Brian's simulation time might be an understimated because of
caching and/or memory access issues.
"""
from brian import *
from time import time

dt = defaultclock.dt

N = 100000 # neurons
p = 10000 # synapses (that's a billion synapses)
F = 2.5 * Hz # firing rate
duration = 1 * second

taum = 20 * ms
Vt = -50 * mV
Vr = -60 * mV
El = -49 * mV
sigma = 1 * mV

eqs = """
dv/dt  = -(v-El)/taum + sigma/taum**.5*xi: volt
"""

p_active = F * dt
Nactive = int(N * p_active) # number of active neurons every time step
active_ones = arange(N) < Nactive
P = NeuronGroup(N, model=eqs, threshold="rand(N)<p_active", reset=Vr, refractory=2 * ms, delay=1 * ms)

# We design a fake connection matrix that always returns the
# same row (thus the simulation time is unchanged but there is no
# memory constraint).
class FakeConstructionMatrix(SparseConstructionMatrix):
    def connection_matrix(self):
        return FakeConnectionMatrix(self, **self.init_kwds)
class FakeConnectionMatrix(SparseConnectionMatrix):
    def get_row(self, i):
        return self.rows[0]

    def get_rows(self, rows):
        return [self.rows[0] for i in rows]

C = Connection(P, P, 'v', structure=FakeConstructionMatrix)
C.connect_random(P[:1], P, sparseness=p * 1. / N, weight=1 * mV)

M = SpikeMonitor(P, record=False)

print "Starting..."
t1 = time()
run(duration)
t2 = time()
print "It took", t2 - t1, "s"
print M.nspikes, "spikes"



from codegen import *

__all__ = ['PythonCodeGenerator']


class PythonCodeGenerator(CodeGenerator):
    def initialisation(self, eqs):
        code = ''
        for i, var in enumerate(eqs._diffeq_names):
            code += var + ' = _S[' + str(i) + ']\n'
        return code

    def single_statement(self, expr):
        return CodeGenerator.single_statement(self, expr).strip()

    def single_expr(self, expr):
        return CodeGenerator.single_expr(self, expr).strip()

    def single_substitute(self, s):
#        if s==0:
#            return 'zeros(num_neurons)'
#        if s==1:
#            return 'ones(num_neurons)'
        return str(s)



'''
Dendrite with 100 compartments
'''
from brian import *
from brian.compartments import *
from brian.library.ionic_currents import *

length = 1 * mm
nseg = 100
dx = length / nseg
Cm = 1 * uF / cm ** 2
gl = 0.02 * msiemens / cm ** 2
diam = 1 * um
area = pi * diam * dx
El = 0 * mV
Ri = 100 * ohm * cm
ra = Ri * 4 / (pi * diam ** 2)

print "Time constant =", Cm / gl
print "Space constant =", .5 * (diam / (gl * Ri)) ** .5

segments = {}
for i in range(nseg):
    segments[i] = MembraneEquation(Cm * area) + leak_current(gl * area, El)

segments[0] += Current('I:nA')

cable = Compartments(segments)
for i in range(nseg - 1):
    cable.connect(i, i + 1, ra * dx)

neuron = NeuronGroup(1, model=cable)
#neuron.vm_0=10*mV
neuron.I_0 = .05 * nA

trace = []
for i in range(10):
    trace.append(StateMonitor(neuron, 'vm_' + str(10 * i), record=True))

run(200 * ms)

for i in range(10):
    plot(trace[i].times / ms, trace[i][0] / mV)
show()



'''
Example with named threshold and reset variables
'''
from brian import *
eqs = '''
dge/dt = -ge/(5*ms) : volt
dgi/dt = -gi/(10*ms) : volt
dx/dt = (ge+gi-(x+49*mV))/(20*ms) : volt
'''
P = NeuronGroup(4000, model=eqs, threshold='x>-50*mV', \
              reset=Refractoriness(-60 * mV, 5 * ms, state='x'))
#P=NeuronGroup(4000,model=eqs,threshold=Threshold(-50*mV,state='x'),\
#              reset=Reset(-60*mV,state='x')) # without refractoriness
P.x = -60 * mV
Pe = P.subgroup(3200)
Pi = P.subgroup(800)
Ce = Connection(Pe, P, 'ge', weight=1.62 * mV, sparseness=0.02)
Ci = Connection(Pi, P, 'gi', weight= -9 * mV, sparseness=0.02)
M = SpikeMonitor(P)
run(1 * second)
raster_plot(M)
show()



'''
Tutorial 2b: Excitatory and inhibitory currents
***********************************************

In this tutorial, we use multiple connections to solve a real problem, how
to implement two types of synapses with excitatory and inhibitory currents
with different time constants.

The scheme
~~~~~~~~~~

The scheme we implement is the following diffential equations:

    | taum dV/dt = -V + ge - gi
    | taue dge/dt = -ge
    | taui dgi/dt = -gi

An excitatory neuron connects to state ge, and an inhibitory neuron connects
to state gi. When an excitatory spike arrives, ge instantaneously increases,
then decays exponentially. Consequently, V will initially but continuously
rise and then fall. Solving these equations, if V(0)=0, ge(0)=g0 corresponding
to an excitatory spike arriving at time 0, and gi(0)=0 then:

    | gi = 0    
    | ge = g0 exp(-t/taue)
    | V = (exp(-t/taum) - exp(-t/taue)) taue g0 / (taum-taue)

We use a very short time constant for the excitatory currents, a longer one
for the inhibitory currents, and an even longer one for the membrane
potential.
'''
from brian import *

taum = 20 * ms
taue = 1 * ms
taui = 10 * ms
Vt = 10 * mV
Vr = 0 * mV

eqs = Equations('''
      dV/dt  = (-V+ge-gi)/taum : volt
      dge/dt = -ge/taue        : volt
      dgi/dt = -gi/taui        : volt
      ''')

'''
Connections
~~~~~~~~~~~

As before, we'll have a group of two neurons under direct control, the first
of which will be excitatory this time, and the second will be inhibitory. To
demonstrate the effect, we'll have two excitatory spikes reasonably close
together, followed by an inhibitory spike later on, and then shortly after
that two excitatory spikes close together.
'''
spiketimes = [(0, 1 * ms), (0, 10 * ms),
              (1, 40 * ms),
              (0, 50 * ms), (0, 55 * ms)]

G1 = SpikeGeneratorGroup(2, spiketimes)
G2 = NeuronGroup(N=1, model=eqs, threshold=Vt, reset=Vr)

C1 = Connection(G1, G2, 'ge')
C2 = Connection(G1, G2, 'gi')
'''
The weights are the same - when we increase ``ge`` the effect on ``V`` is excitatory
and when we increase ``gi`` the effect on ``V`` is inhibitory.
'''
C1[0, 0] = 3 * mV
C2[1, 0] = 3 * mV
'''
We set up monitors and run as normal.
'''
Mv = StateMonitor(G2, 'V', record=True)
Mge = StateMonitor(G2, 'ge', record=True)
Mgi = StateMonitor(G2, 'gi', record=True)

run(100 * ms)
'''
This time we do something a little bit different when plotting it. We want
a plot with two subplots, the top one will show ``V``, and the bottom one will
show both ``ge`` and ``gi``. We use the ``subplot`` command from pylab which mimics the
same command from Matlab.
'''
figure()
subplot(211)
plot(Mv.times, Mv[0])
subplot(212)
plot(Mge.times, Mge[0])
plot(Mgi.times, Mgi[0])
show()
'''
.. image:: images/tutorials/2b.jpg

The top figure shows the voltage trace, and the bottom figure shows ``ge`` in
blue and ``gi`` in green. You can see that although the inhibitory and
excitatory weights are the same, the inhibitory current is much more
powerful. This is because the effect of ``ge`` or ``gi`` on ``V`` is related to the
integral of the differential equation for those variables, and ``gi`` decays
much more slowly than ``ge``. Thus the size of the negative deflection at
40 ms is much bigger than the excitatory ones, and even the double
excitatory spike after the inhibitory one can't cancel it out.

In the next part of this tutorial, we set up our first serious network,
with 4000 neurons, excitatory and inhibitory.

Exercises
~~~~~~~~~

1. Try changing the parameters and spike times to get a feel for how it
   works.
2. Try an equivalent implementation with the equation taum dV/dt = -V+ge+gi
3. Verify that the differential equation has been solved correctly.

Solutions
~~~~~~~~~

Solution for 2:

Simply use the line ``C2[1,0] = -3*mV`` to get the same effect.

Solution for 3:

First, set up the situation we described at the top for which we
already know the solution of the differential equations, by changing
the spike times as follows::

    spiketimes = [(0,0*ms)]

Now we compute what the values ought to be as follows::

    t = Mv.times
    Vpredicted = (exp(-t/taum) - exp(-t/taue))*taue*(3*mV) / (taum-taue)

Now we can compute the difference between the predicted and actual values::

    Vdiff = abs(Vpredicted - Mv[0])

This should be zero::

    print max(Vdiff)

Sure enough, it's as close as you can expect on a computer. When I run this
it gives me the value 1.3 aV, which is 1.3 * 10^-18 volts, i.e. effectively
zero given the finite precision of the calculations involved. 
'''



'''
Bug with insert_spikes, filed by Antoine Bergel on 4 Nov 2010
'''
from brian import *

G=NeuronGroup(1,model='dv/dt=100*Hz:1',threshold=1,reset=0)
M = SpikeMonitor(G)
Mv = StateMonitor(G, 'v', record=True)

run(100*ms)
print Mv.values.shape
Mv.insert_spikes(M,5)

plot(Mv.times/ms, Mv[0])
show()



from brian import *
set_global_preferences(usenewbrianhears=True,
                       useweave=False)
from brian.hears import *

hrtfdb = IRCAM_LISTEN(r'D:\HRTF\IRCAM')
subject = 1002
hrtfset = hrtfdb.load_subject(subject)
hrtf = hrtfset.hrtf[0]

x = Sound(randn(1000, 1), samplerate=44100*Hz)
y = hrtf.apply(x)

subplot(211)
plot(x)
subplot(212)
plot(y)
show()
